﻿/*****************************************************************************
*
* Copyright 2013 Altera Corporation. All Rights Reserved.
* 
* Redistribution and use in source and binary forms, with or without
* modification, are permitted provided that the following conditions are met:
* 
* 1. Redistributions of source code must retain the above copyright notice,
* this list of conditions and the following disclaimer.
* 
* 2. Redistributions in binary form must reproduce the above copyright notice,
* this list of conditions and the following disclaimer in the documentation
* and/or other materials provided with the distribution.
* 
* 3. Neither the name of the copyright holder nor the names of its contributors
* may be used to endorse or promote products derived from this software without
* specific prior written permission.
* 
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
* AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
* IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
* ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
* LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
* CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
* SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
* INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
* CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
* ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
* POSSIBILITY OF SUCH DAMAGE.
* 
*****************************************************************************/
/********************************************************************************//*!
 * @file hwlib.c 
 * 
 * @note  'SoC EDS'のexampleに収録された同名ファイルから必要な処理だけを残してサンプル用処理の呼び出しを新規追加。 
 * 
 * @date <b> History: \<Date\> \<Rev\> \<Keyword\> \<Details\> </b>
 * @date 2013/12/02  ---  #6287  新規作成. 
 * @date 2014/01/24  ---  -----  体裁修正（コメント整理、関数等のラベル接頭辞を変更). 
 * @date 2015/07/03  ---  #20263  v15.0対応(v15.0付属のサンプルをベースに再生成)/文字コード変更(UTF-8) 他. 
 **//*******************************************************************************/

/*
 * $Id: //depot/embedded/rel/15.0/examples/software/Altera-SoCFPGA-HardwareLib-FPGA-CV-GNU/hwlib.c#1 $
 */

#include <stdio.h>
#include <inttypes.h>
#include "alt_address_space.h"
#include "alt_bridge_manager.h"
#include "alt_clock_manager.h"
#include "alt_dma.h"
#include "alt_fpga_manager.h"
#include "socal/socal.h"

#include "alt_hps_detect.h"
#include "alt_printf.h"

#include "socal/hps.h"
#include "socal/alt_fpgamgr.h"
#include "util_time_measurement.h"
#include "alt_interrupt.h"
#include "alt_system_manager.h"
#include "alt_globaltmr.h"

/// prototype ///
void sample_fpgamanager_setting_gic(void);
void sample_fpgamanager_callback(uint32_t icciar, void *context);

#define ALT_FPGA_MONMASK_ALL	((ALT_FPGA_MON_nSTATUS) | (ALT_FPGA_MON_CONF_DONE) | (ALT_FPGA_MON_INIT_DONE) |\
								 (ALT_FPGA_MON_CRC_ERROR) | (ALT_FPGA_MON_CVP_CONF_DONE) | (ALT_FPGA_MON_PR_READY) |\
								 (ALT_FPGA_MON_PR_ERROR) | (ALT_FPGA_MON_PR_DONE) | (ALT_FPGA_MON_nCONFIG_PIN) |\
								 (ALT_FPGA_MON_nSTATUS_PIN) | (ALT_FPGA_MON_CONF_DONE_PIN) | (ALT_FPGA_MON_FPGA_POWER_ON))

#define ALT_FPGA_MONMASK_IRQSRC	((ALT_FPGA_MON_CONF_DONE) | (ALT_FPGA_MON_INIT_DONE) |\
								 (ALT_FPGA_MON_CRC_ERROR) | (ALT_FPGA_MON_CVP_CONF_DONE) | (ALT_FPGA_MON_PR_READY) |\
								 (ALT_FPGA_MON_PR_ERROR) | (ALT_FPGA_MON_PR_DONE) |\
								 (ALT_FPGA_MON_CONF_DONE_PIN) | (ALT_FPGA_MON_FPGA_POWER_ON))

typedef struct {
	bool		flag;			// Interrupt Detect Flag (true:Detected / false:Non-Detect)
	uint32_t	globaltmrlow32;	// Low 32bit Data of Global Timer Counter
	uint32_t	opt1;			// Optional Info1 (EXT_PORTA)
	uint32_t	opt2;			// Optional Info2 (INTSTAT)
} sample_isrloginfo_t;
sample_isrloginfo_t	isrlog_FPGA_MAN_IRQ_fallEdge[12];
sample_isrloginfo_t	isrlog_FPGA_MAN_IRQ_riseEdge[12];
uint32_t	isrlog_lastintstat;

void sample_fpgamanager_setting_gic(void)
{
	ALT_STATUS_CODE result_code;
	uint32_t ui32temp = 0;
	ALT_INT_TRIGGER_t trigger_type = ALT_INT_TRIGGER_NA;

	// Print initial value (GIC: ALT_INT_INTERRUPT_CLKMGR_IRQ)
	/****************************************************************************************/
	/* Usage: ALT_STATUS_CODE alt_int_dist_is_secure(ALT_INT_INTERRUPT_t int_id);	*/
	/* Usage: ALT_STATUS_CODE alt_int_dist_is_enabled(ALT_INT_INTERRUPT_t int_id);	*/
	/* Usage: ALT_STATUS_CODE alt_int_dist_is_pending(ALT_INT_INTERRUPT_t int_id);	*/
	/* Usage: ALT_STATUS_CODE alt_int_dist_is_active(ALT_INT_INTERRUPT_t int_id);	*/
	/* Usage: ALT_STATUS_CODE alt_int_dist_priority_get(ALT_INT_INTERRUPT_t int_id, uint32_t* priority);	*/
	/* Usage: ALT_STATUS_CODE alt_int_dist_target_get(ALT_INT_INTERRUPT_t int_id, alt_int_cpu_target_t* target);	*/
	/* Usage: ALT_STATUS_CODE alt_int_dist_trigger_get(ALT_INT_INTERRUPT_t int_id, ALT_INT_TRIGGER_t* trigger_type);	*/
	/****************************************************************************************/
	PRINT_WITH_GLOBALTMR("-- HWLib test : alt_int_dist_is_secure() --\n");
	ALT_PRINTF("arg1=ALT_INT_INTERRUPT_FPGA_MAN_IRQ --> result=%d\n", (int)alt_int_dist_is_secure(ALT_INT_INTERRUPT_FPGA_MAN_IRQ));
	PRINT_WITH_GLOBALTMR("-- HWLib test : alt_int_dist_is_enabled() --\n");
	ALT_PRINTF("arg1=ALT_INT_INTERRUPT_FPGA_MAN_IRQ --> result=%d\n", (int)alt_int_dist_is_enabled(ALT_INT_INTERRUPT_FPGA_MAN_IRQ));
	PRINT_WITH_GLOBALTMR("-- HWLib test : alt_int_dist_is_pending() --\n");
	ALT_PRINTF("arg1=ALT_INT_INTERRUPT_FPGA_MAN_IRQ --> result=%d\n", (int)alt_int_dist_is_pending(ALT_INT_INTERRUPT_FPGA_MAN_IRQ));
	PRINT_WITH_GLOBALTMR("-- HWLib test : alt_int_dist_is_active() --\n");
	ALT_PRINTF("arg1=ALT_INT_INTERRUPT_FPGA_MAN_IRQ --> result=%d\n", (int)alt_int_dist_is_active(ALT_INT_INTERRUPT_FPGA_MAN_IRQ));
	PRINT_WITH_GLOBALTMR("-- HWLib test : alt_int_dist_priority_get() --\n");
	ALT_PRINTF("arg1=ALT_INT_INTERRUPT_FPGA_MAN_IRQ --> result=%d out=0x%08X\n", (int)alt_int_dist_priority_get(ALT_INT_INTERRUPT_FPGA_MAN_IRQ,&ui32temp), (int)ui32temp);
	PRINT_WITH_GLOBALTMR("-- HWLib test : alt_int_dist_target_get() --\n");
	ALT_PRINTF("arg1=ALT_INT_INTERRUPT_FPGA_MAN_IRQ --> result=%d out=0x%08X\n", (int)alt_int_dist_target_get(ALT_INT_INTERRUPT_FPGA_MAN_IRQ,&ui32temp), (int)ui32temp);
	PRINT_WITH_GLOBALTMR("-- HWLib test : alt_int_dist_trigger_get() --\n");
	ALT_PRINTF("arg1=ALT_INT_INTERRUPT_FPGA_MAN_IRQ --> result=%d out=%d\n", (int)alt_int_dist_trigger_get(ALT_INT_INTERRUPT_FPGA_MAN_IRQ,&trigger_type), (int)trigger_type);
	ALT_PRINTF("\n");

	/* Usage: ALT_STATUS_CODE alt_int_isr_register(ALT_INT_INTERRUPT_t int_id, alt_int_callback_t callback, void* context);	*/
	/*  typedef void(* alt_int_callback_t)(uint32_t icciar, void *context) */
	PRINT_WITH_GLOBALTMR("-- HWLib test : alt_int_isr_register() --\n");
	result_code = alt_int_isr_register(ALT_INT_INTERRUPT_FPGA_MAN_IRQ, sample_fpgamanager_callback, NULL);
	if(result_code != ALT_E_SUCCESS){
		ALT_PRINTF("ERROR!!: alt_int_isr_register() result=%d\n", (int)result_code);
	} else {
		ALT_PRINTF("SUCCESS!!\n");
	}
	ALT_PRINTF("\n");

	/* Usage: ALT_STATUS_CODE alt_int_dist_target_set(ALT_INT_INTERRUPT_t int_id, alt_int_cpu_target_t target);	*/
	PRINT_WITH_GLOBALTMR("-- HWLib test : alt_int_dist_target_set() --\n");
	result_code = alt_int_dist_target_set(ALT_INT_INTERRUPT_FPGA_MAN_IRQ, 1);		// arg2=targetCPU (Only Core#0)
	if(result_code != ALT_E_SUCCESS)
	{
		ALT_PRINTF("ERROR!!: alt_int_dist_target_set() result=%d\n", (int)result_code);
	} else {
		ALT_PRINTF("SUCCESS!!\n");
	}
	ALT_PRINTF("\n");

	/* Usage: ALT_STATUS_CODE alt_int_dist_trigger_set(ALT_INT_INTERRUPT_t int_id, ALT_INT_TRIGGER_t trigger_type);	*/
	PRINT_WITH_GLOBALTMR("-- HWLib test : alt_int_dist_trigger_set() --\n");
	result_code = alt_int_dist_trigger_set(ALT_INT_INTERRUPT_FPGA_MAN_IRQ, ALT_INT_TRIGGER_EDGE);
	if(result_code != ALT_E_SUCCESS){
		ALT_PRINTF("ERROR!!: alt_int_dist_trigger_set() result=%d\n", (int)result_code);
	} else {
		ALT_PRINTF("SUCCESS!!\n");
	}
	ALT_PRINTF("\n");

	/* Usage: ALT_STATUS_CODE alt_int_dist_priority_set(ALT_INT_INTERRUPT_t int_id, uint32_t priority);	*/
	PRINT_WITH_GLOBALTMR("-- HWLib test : alt_int_dist_priority_set() --\n");
	result_code = alt_int_dist_priority_set(ALT_INT_INTERRUPT_FPGA_MAN_IRQ, 99);
	if(result_code != ALT_E_SUCCESS){
		ALT_PRINTF("ERROR!!: alt_int_dist_priority_set() result=%d\n", (int)result_code);
	} else {
		ALT_PRINTF("SUCCESS!!\n");
	}
	ALT_PRINTF("\n");

	/* Usage: ALT_STATUS_CODE alt_int_dist_enable(ALT_INT_INTERRUPT_t int_id);	*/
	PRINT_WITH_GLOBALTMR("-- HWLib test : alt_int_dist_enable() --\n");
	result_code = alt_int_dist_enable(ALT_INT_INTERRUPT_FPGA_MAN_IRQ);
	if(result_code != ALT_E_SUCCESS){
		ALT_PRINTF("ERROR!!: alt_int_dist_enable() result=%d\n", (int)result_code);
	} else {
		ALT_PRINTF("SUCCESS!!\n");
	}
	ALT_PRINTF("\n");

	/* Usage: ALT_STATUS_CODE alt_int_dist_secure_enable(ALT_INT_INTERRUPT_t int_id);	*/
	PRINT_WITH_GLOBALTMR("-- HWLib test : alt_int_dist_secure_enable() --\n");
	result_code = alt_int_dist_secure_enable(ALT_INT_INTERRUPT_FPGA_MAN_IRQ);
	if(result_code != ALT_E_SUCCESS){
		ALT_PRINTF("ERROR!!: alt_int_dist_enable() result=%d\n", (int)result_code);
	} else {
		ALT_PRINTF("SUCCESS!!\n");
	}
	ALT_PRINTF("\n");

	/* Usage: ALT_STATUS_CODE alt_int_dist_pending_clear(ALT_INT_INTERRUPT_t int_id);	*/
	PRINT_WITH_GLOBALTMR("-- HWLib test : alt_int_dist_pending_clear() --\n");
	result_code = alt_int_dist_pending_clear(ALT_INT_INTERRUPT_FPGA_MAN_IRQ);
	if(result_code != ALT_E_SUCCESS)
	{
		ALT_PRINTF("ERROR!!: alt_int_dist_pending_clear() result=%d\n", (int)result_code);
	} else {
		ALT_PRINTF("SUCCESS!!\n");
	}

    // Interrupt Mask Setting set to All Disable.
    alt_write_word(ALT_FPGAMGR_MON_GPIO_INTMSK_ADDR, (uint32_t)(ALT_FPGA_MONMASK_ALL ^ ALT_FPGA_MONMASK_IRQSRC));
    // Interrupt Detect Setting set to Rising-Edge.
    alt_write_word(ALT_FPGAMGR_MON_GPIO_INTTYPE_LEVEL_ADDR, (uint32_t)ALT_FPGA_MONMASK_ALL);
    alt_write_word(ALT_FPGAMGR_MON_GPIO_INT_POL_ADDR, (uint32_t)ALT_FPGA_MONMASK_ALL);
	// Interrupt Source all reset.
    alt_write_word(ALT_FPGAMGR_MON_GPIO_PORTA_EOI_ADDR, (uint32_t)ALT_FPGA_MONMASK_ALL);

	return;
}

void sample_fpgamanager_callback(uint32_t icciar, void *context)
{
	uint32_t	fpgamon_status;
	uint32_t	temp_changestat;
	uint32_t	temp_fallingstat;
	uint32_t	temp_risingstat;
	uint32_t	temp_maskbit = 0x00000001;
	sample_isrloginfo_t*	isrlog_p;

	// read Interraupt Source Status
	fpgamon_status = alt_read_word(ALT_FPGAMGR_MON_GPIO_INTSTAT_ADDR);

	// clear Interrupt Source Status
    alt_write_word(ALT_FPGAMGR_MON_GPIO_PORTA_EOI_ADDR, (uint32_t)fpgamon_status);

	/* Usage: ALT_STATUS_CODE alt_int_dist_pending_clear(ALT_INT_INTERRUPT_t int_id);	*/
	(void)alt_int_dist_pending_clear(ALT_INT_INTERRUPT_FPGA_MAN_IRQ);

	//ALT_PRINTF("[INTERRUPT]FPGA Manager EXT_PORTA=0x%08X, INTSTAT=0x%08X\n", (int)alt_fpga_mon_status_get(), (int)fpgamon_status);
	// Recording Interrupt Detect Info for Console out on Normal Routine.
	temp_changestat = (isrlog_lastintstat ^ fpgamon_status) & ALT_FPGA_MONMASK_ALL;
	temp_fallingstat = temp_changestat & (~fpgamon_status);
	temp_risingstat = temp_changestat & fpgamon_status;
	for(int i=0; i<12; i++, temp_maskbit<<=1){
		if((temp_fallingstat & temp_maskbit) != 0){
			isrlog_p = &isrlog_FPGA_MAN_IRQ_fallEdge[i];
			isrlog_p->globaltmrlow32 = alt_globaltmr_counter_get_low32();
			isrlog_p->opt1 = alt_fpga_mon_status_get();
			isrlog_p->opt2 = fpgamon_status;
			isrlog_p->flag = true;
		}
		if((temp_risingstat & temp_maskbit) != 0){
			isrlog_p = &isrlog_FPGA_MAN_IRQ_riseEdge[i];
			isrlog_p->globaltmrlow32 = alt_globaltmr_counter_get_low32();
			isrlog_p->opt1 = alt_fpga_mon_status_get();
			isrlog_p->opt2 = fpgamon_status;
			isrlog_p->flag = true;
		}
	}

	isrlog_lastintstat = fpgamon_status;

	return;
}


/* enable semihosting with gcc by defining an __auto_semihosting symbol */
int __auto_semihosting;

ALT_STATUS_CODE socfpga_dma_setup(ALT_DMA_CHANNEL_t * allocated)
{
    PRINT_WITH_GLOBALTMR("INFO: Setup DMA System ...\n");

    ALT_STATUS_CODE status = ALT_E_SUCCESS;

    if (status == ALT_E_SUCCESS)
    {
        // Configure everything as defaults.

        ALT_DMA_CFG_t dma_config;
        dma_config.manager_sec = ALT_DMA_SECURITY_DEFAULT;
        for (int i = 0; i < 8; ++i)
        {
            dma_config.irq_sec[i] = ALT_DMA_SECURITY_DEFAULT;
        }
        for (int i = 0; i < 32; ++i)
        {
            dma_config.periph_sec[i] = ALT_DMA_SECURITY_DEFAULT;
        }
        for (int i = 0; i < 4; ++i)
        {
            dma_config.periph_mux[i] = ALT_DMA_PERIPH_MUX_DEFAULT;
        }

        status = alt_dma_init(&dma_config);
        if (status != ALT_E_SUCCESS)
        {
            PRINT_WITH_GLOBALTMR("ERROR: alt_dma_init() failed.\n");
        }
    }

    // Allocate the DMA channel

    if (status == ALT_E_SUCCESS)
    {
        status = alt_dma_channel_alloc_any(allocated);
        if (status != ALT_E_SUCCESS)
        {
            PRINT_WITH_GLOBALTMR("ERROR: alt_dma_channel_alloc_any() failed.\n");
        }
        else
        {
            PRINT_WITH_GLOBALTMR("INFO: Channel %d allocated.\n", *allocated);
        }
    }

    // Verify channel state

    if (status == ALT_E_SUCCESS)
    {
        ALT_DMA_CHANNEL_STATE_t state;
        status = alt_dma_channel_state_get(*allocated, &state);
        if (status == ALT_E_SUCCESS)
        {
            if (state != ALT_DMA_CHANNEL_STATE_STOPPED)
            {
                PRINT_WITH_GLOBALTMR("ERROR: Bad initial channel state.\n");
                status = ALT_E_ERROR;
            }
        }
    }

    if (status == ALT_E_SUCCESS)
    {
        PRINT_WITH_GLOBALTMR("INFO: Setup of DMA successful.\n\n");
    }
    else
    {
        PRINT_WITH_GLOBALTMR("ERROR: Setup of DMA failed. [status = %" PRIi32 "].\n\n", status);
    }

    return status;
}

void socfpga_dma_cleanup(ALT_DMA_CHANNEL_t channel)
{
    PRINT_WITH_GLOBALTMR("INFO: Cleaning up DMA System ...\n");

    if (alt_dma_channel_free(channel) != ALT_E_SUCCESS)
    {
        PRINT_WITH_GLOBALTMR("WARN: alt_dma_channel_free() returned non-SUCCESS.\n");
    }

    if (alt_dma_uninit() != ALT_E_SUCCESS)
    {
        PRINT_WITH_GLOBALTMR("WARN: alt_dma_uninit() returned non-SUCCESS.\n");
    }
}

ALT_STATUS_CODE socfpga_fpga_setup_dma(const void * fpga_image, size_t fpga_image_size, 
                                       ALT_DMA_CHANNEL_t dma_channel)
{
	ALT_STATUS_CODE result_code;

    PRINT_WITH_GLOBALTMR("INFO: Setup FPGA System ...\n");

    ALT_STATUS_CODE status = ALT_E_SUCCESS;

    if (status == ALT_E_SUCCESS)
    {
        status = alt_fpga_init();
    }

    // Verify power is on
    if (status == ALT_E_SUCCESS)
    {
        if (alt_fpga_state_get() == ALT_FPGA_STATE_POWER_OFF)
        {
            PRINT_WITH_GLOBALTMR("ERROR: FPGA Monitor reports FPGA is powered off.\n");
            status = ALT_E_ERROR;
        }
    }

    // Take control of the FPGA CB
    if (status == ALT_E_SUCCESS)
    {
        status = alt_fpga_control_enable();
    }

    /* Usage: bool alt_fpga_control_is_enabled(void);	 */
	PRINT_WITH_GLOBALTMR("-- HWLib test : alt_fpga_control_is_enabled() [result=%d] --\n", (int)alt_fpga_control_is_enabled());
	/* Usage: uint32_t alt_fpga_mon_status_get(void);	 */
	PRINT_WITH_GLOBALTMR("-- HWLib test : alt_fpga_mon_status_get() [result=0x%08X] --\n", (int)alt_fpga_mon_status_get());
	/* Usage: ALT_STATUS_CODE alt_fgpa_reset_assert(void);	 */
	PRINT_WITH_GLOBALTMR("-- HWLib test : alt_fgpa_reset_assert() --\n");
	result_code = alt_fgpa_reset_assert();
	if(result_code != ALT_E_SUCCESS)
	{
		ALT_PRINTF("ERROR!!: alt_fgpa_reset_assert() result=%d\n", (int)result_code);
	} else {
		ALT_PRINTF("SUCCESS!!\n");
	}

    // Verify the MSELs are appropriate for the type of image we're using.
    if (status == ALT_E_SUCCESS)
    {
        // From experimentation, a compressed image is ~2.3 MiB an uncompressed is ~6.8 MiB.
        // Thus if it's smaller than 3 MiB, consider it compressed.
        bool compressed = (fpga_image_size < 3 * 1024 * 1024);

        ALT_FPGA_CFG_MODE_t mode = alt_fpga_cfg_mode_get();
        switch (mode)
        {
            // Compressed image modes
        case ALT_FPGA_CFG_MODE_PP32_FAST_AESOPT_DC:
        case ALT_FPGA_CFG_MODE_PP32_SLOW_AESOPT_DC:
        case ALT_FPGA_CFG_MODE_PP16_FAST_AESOPT_DC:
        case ALT_FPGA_CFG_MODE_PP16_SLOW_AESOPT_DC:
            if (compressed)
            {
                PRINT_WITH_GLOBALTMR("INFO: MSEL [%d] configured correctly for compressed FPGA image.\n", mode);
            }
            else
            {
                PRINT_WITH_GLOBALTMR("WARNING: MSEL [%d] likely configured incorrectly for uncompressed FPGA image.\n", mode);
                PRINT_WITH_GLOBALTMR("WARNING: Because of this, FPGA programming will probably fail.\n");
            }
            break;

            // Uncompressed image modes
        case ALT_FPGA_CFG_MODE_PP16_FAST_NOAES_NODC:
        case ALT_FPGA_CFG_MODE_PP16_FAST_AES_NODC:
        case ALT_FPGA_CFG_MODE_PP16_SLOW_NOAES_NODC:
        case ALT_FPGA_CFG_MODE_PP16_SLOW_AES_NODC:
        case ALT_FPGA_CFG_MODE_PP32_FAST_NOAES_NODC:
        case ALT_FPGA_CFG_MODE_PP32_FAST_AES_NODC:
        case ALT_FPGA_CFG_MODE_PP32_SLOW_NOAES_NODC:
        case ALT_FPGA_CFG_MODE_PP32_SLOW_AES_NODC:
            if (compressed)
            {
                PRINT_WITH_GLOBALTMR("WARNING: MSEL [%d] likely configured incorrectly for compressed FPGA image.\n", mode);
                PRINT_WITH_GLOBALTMR("WARNING: Because of this, FPGA programming will probably fail.\n");
            }
            else
            {
                PRINT_WITH_GLOBALTMR("INFO: MSEL [%d] configured correctly for uncompressed FPGA image.\n", mode);
            }
            break;
                
            // Invalid modes
        default:
            PRINT_WITH_GLOBALTMR("ERROR: Incompatible MSEL [%d] set. Cannot continue with FPGA programming.\n", mode);
            status = ALT_E_ERROR;
            break;
        }
    }

    // Program the FPGA
    if (status == ALT_E_SUCCESS)
    {
        // Try the full configuration a few times.
        const uint32_t full_config_retry = 5;
        for (uint32_t i = 0; i < full_config_retry; ++i)
        {
            util_time_record_start_point(MP_SETUP_FPGA_CONFIGURE + i);
#if ALT_FPGA_ENABLE_DMA_SUPPORT == 1
            status = alt_fpga_configure_dma(fpga_image, fpga_image_size, dma_channel);
#else
            status = alt_fpga_configure(fpga_image, fpga_image_size);
#endif
            util_time_record_end_point(MP_SETUP_FPGA_CONFIGURE + i);
            if (status == ALT_E_SUCCESS)
            {
                /* Usage: bool alt_fpga_control_is_enabled(void);	 */
            	PRINT_WITH_GLOBALTMR("-- HWLib test : alt_fpga_control_is_enabled() [result=%d] --\n", (int)alt_fpga_control_is_enabled());
            	/* Usage: uint32_t alt_fpga_mon_status_get(void);	 */
            	PRINT_WITH_GLOBALTMR("-- HWLib test : alt_fpga_mon_status_get() [result=0x%08X] --\n", (int)alt_fpga_mon_status_get());
            	/* Usage: ALT_STATUS_CODE alt_fgpa_reset_deassert(void);	 */
            	PRINT_WITH_GLOBALTMR("-- HWLib test : alt_fgpa_reset_deassert() --\n");
            	result_code = alt_fgpa_reset_deassert();
            	if(result_code != ALT_E_SUCCESS)
            	{
            		ALT_PRINTF("ERROR!!: alt_fgpa_reset_deassert() result=%d\n", (int)result_code);
            	} else {
            		ALT_PRINTF("SUCCESS!!\n");
            	}

            	/* Usage: ALT_STATUS_CODE alt_int_dist_pending_clear(ALT_INT_INTERRUPT_t int_id);	*/
            	PRINT_WITH_GLOBALTMR("-- HWLib test : alt_int_dist_pending_clear() --\n");
            	result_code = alt_int_dist_pending_clear(ALT_INT_INTERRUPT_FPGA_MAN_IRQ);
            	if(result_code != ALT_E_SUCCESS)
            	{
            		ALT_PRINTF("ERROR!!: alt_int_dist_pending_clear() result=%d\n", (int)result_code);
            	} else {
            		ALT_PRINTF("SUCCESS!!\n");
            	}

            	/* Usage: uint32_t alt_fpga_mon_status_get(void);	 */
            	PRINT_WITH_GLOBALTMR("-- HWLib test : alt_fpga_mon_status_get() [result=0x%08X] --\n", (int)alt_fpga_mon_status_get());

                break;
            }
        }
    }

    if (status == ALT_E_SUCCESS)
    {
        PRINT_WITH_GLOBALTMR("INFO: Setup of FPGA successful.\n\n");
    }
    else
    {
        PRINT_WITH_GLOBALTMR("ERROR: Setup of FPGA failed. [status = %" PRIi32 "].\n\n", status);
    }

    return status;
}

void socfpga_fpga_cleanup(void)
{
    PRINT_WITH_GLOBALTMR("INFO: Cleanup of FPGA ...\n");

    if (alt_fpga_control_disable() != ALT_E_SUCCESS)
    {
        PRINT_WITH_GLOBALTMR("WARN: alt_fpga_control_disable() returned non-SUCCESS.\n");
    }

    if (alt_fpga_uninit() != ALT_E_SUCCESS)
    {
        PRINT_WITH_GLOBALTMR("WARN: alt_fpga_uninit() returned non-SUCCESS.\n");
    }
}

ALT_STATUS_CODE socfpga_bridge_setup(ALT_BRIDGE_t bridge)
{
    PRINT_WITH_GLOBALTMR("INFO: Setup Bridge [%d] ...\n", bridge);

    ALT_STATUS_CODE status = ALT_E_SUCCESS;

    if (status == ALT_E_SUCCESS)
    {
        status = alt_bridge_init(bridge, NULL, NULL);
    }

    if (status == ALT_E_SUCCESS)
    {
        status = alt_addr_space_remap(ALT_ADDR_SPACE_MPU_ZERO_AT_BOOTROM,
                                      ALT_ADDR_SPACE_NONMPU_ZERO_AT_OCRAM,
                                      ALT_ADDR_SPACE_H2F_ACCESSIBLE,
                                      ALT_ADDR_SPACE_LWH2F_ACCESSIBLE);
    }

    if (status == ALT_E_SUCCESS)
    {
        PRINT_WITH_GLOBALTMR("INFO: Setup of Bridge [%d] successful.\n\n", bridge);
    }
    else
    {
        PRINT_WITH_GLOBALTMR("ERROR: Setup of Bridge [%d] failed. [status = %" PRIi32 "].\n\n", bridge, status);
    }

    return status;
}

void socfpga_bridge_cleanup(ALT_BRIDGE_t bridge)
{
    PRINT_WITH_GLOBALTMR("INFO: Cleanup of Bridge [%d] ...\n", bridge);

    if (alt_bridge_uninit(bridge, NULL, NULL) != ALT_E_SUCCESS)
    {
        PRINT_WITH_GLOBALTMR("WARN: alt_bridge_uninit() returned non-SUCCESS.\n");
    }
}

ALT_STATUS_CODE socfpga_bridge_io(void)
{
    PRINT_WITH_GLOBALTMR("INFO: Demostrating IO across bridge ...\n");

    const uint32_t ALT_LWFPGA_BASE         = 0xFF200000;
    const uint32_t ALT_LWFPGA_SYSID_OFFSET = 0x00010000;
    const uint32_t ALT_LWFPGA_LED_OFFSET   = 0x00010040;

    // Attempt to read the system ID peripheral
    uint32_t sysid = alt_read_word(ALT_LWFPGA_BASE + ALT_LWFPGA_SYSID_OFFSET);
    PRINT_WITH_GLOBALTMR("INFO: LWFPGA Slave => System ID Peripheral value = 0x%" PRIx32 ".\n", sysid);

    // Attempt to toggle the 4 LEDs
    const uint32_t bits = 4;
    PRINT_WITH_GLOBALTMR("INFO: Toggling LEDs ...\n");
    for (uint32_t i = 0; i < (1 << bits); ++i)
    {
        // Use Gray code ... for fun!
        // http://en.wikipedia.org/wiki/Gray_code
        uint32_t gray = (i >> 1) ^ i;

        alt_write_word(ALT_LWFPGA_BASE + ALT_LWFPGA_LED_OFFSET, gray);

        PRINT_WITH_GLOBALTMR("INFO: Gray code(i=0x%x) => 0x%x [", (unsigned int)i, (unsigned int)gray);

        for (uint32_t j = 0; j < bits; ++j)
        {
            ALT_PRINTF("%c", (gray & (1 << (bits - j - 1))) ? '1' : '0');
        }

        ALT_PRINTF("].\n");
    }

    // Reset the LEDs to all on
    alt_write_word(ALT_LWFPGA_BASE + ALT_LWFPGA_LED_OFFSET, 0);

    PRINT_WITH_GLOBALTMR("INFO: LEDs should have blinked.\n\n");

    return ALT_E_SUCCESS;
}


static void sample_systemmanager_print_is_enable(void)
{
	/* Usage: ALT_STATUS_CODE alt_fpga_interface_is_enabled  ( ALT_FPGA_INTERFACE_t  intfc ); 	*/
	PRINT_WITH_GLOBALTMR("-- HWLib test : alt_fpga_interface_is_enabled() --\n");
	ALT_PRINTF("- ALT_FPGA_INTERFACE_GLOBAL ...... result=%d\n", (int)alt_fpga_interface_is_enabled(ALT_FPGA_INTERFACE_GLOBAL));
	ALT_PRINTF("- ALT_FPGA_INTERFACE_RESET_REQ ... result=%d\n", (int)alt_fpga_interface_is_enabled(ALT_FPGA_INTERFACE_RESET_REQ));
	ALT_PRINTF("- ALT_FPGA_INTERFACE_JTAG_ENABLE . result=%d\n", (int)alt_fpga_interface_is_enabled(ALT_FPGA_INTERFACE_JTAG_ENABLE));
	ALT_PRINTF("- ALT_FPGA_INTERFACE_CONFIG_IO ... result=%d\n", (int)alt_fpga_interface_is_enabled(ALT_FPGA_INTERFACE_CONFIG_IO));
	ALT_PRINTF("- ALT_FPGA_INTERFACE_BSCAN ....... result=%d\n", (int)alt_fpga_interface_is_enabled(ALT_FPGA_INTERFACE_BSCAN));
	ALT_PRINTF("- ALT_FPGA_INTERFACE_TRACE ....... result=%d\n", (int)alt_fpga_interface_is_enabled(ALT_FPGA_INTERFACE_TRACE));
	ALT_PRINTF("- ALT_FPGA_INTERFACE_DBG_APB ..... result=%d\n", (int)alt_fpga_interface_is_enabled(ALT_FPGA_INTERFACE_DBG_APB));
	ALT_PRINTF("- ALT_FPGA_INTERFACE_STM ......... result=%d\n", (int)alt_fpga_interface_is_enabled(ALT_FPGA_INTERFACE_STM));
	ALT_PRINTF("- ALT_FPGA_INTERFACE_CTI ......... result=%d\n", (int)alt_fpga_interface_is_enabled(ALT_FPGA_INTERFACE_CTI));
	ALT_PRINTF("- ALT_FPGA_INTERFACE_EMAC0 ....... result=%d\n", (int)alt_fpga_interface_is_enabled(ALT_FPGA_INTERFACE_EMAC0));
	ALT_PRINTF("- ALT_FPGA_INTERFACE_EMAC1 ....... result=%d\n", (int)alt_fpga_interface_is_enabled(ALT_FPGA_INTERFACE_EMAC1));
	ALT_PRINTF("- ALT_FPGA_INTERFACE_SPIM0 ....... result=%d\n", (int)alt_fpga_interface_is_enabled(ALT_FPGA_INTERFACE_SPIM0));
	ALT_PRINTF("- ALT_FPGA_INTERFACE_SPIM0 ....... result=%d\n", (int)alt_fpga_interface_is_enabled(ALT_FPGA_INTERFACE_SPIM1));
	ALT_PRINTF("- ALT_FPGA_INTERFACE_NAND ........ result=%d\n", (int)alt_fpga_interface_is_enabled(ALT_FPGA_INTERFACE_NAND));
	ALT_PRINTF("- ALT_FPGA_INTERFACE_SDMMC ....... result=%d\n", (int)alt_fpga_interface_is_enabled(ALT_FPGA_INTERFACE_SDMMC));
	ALT_PRINTF("\n");

	return;
}

static void sample_fpgamanager_print_fpgamgrregs_mon(void)
{
	PRINT_WITH_GLOBALTMR("--- Print Configuration Monitor (MON) Registers ----\n");
	ALT_PRINTF("ALT_MON_GPIO_INTEN value .......... 0x%08X\n", (int)alt_read_word(ALT_FPGAMGR_MON_GPIO_INTEN_ADDR));
	ALT_PRINTF("ALT_MON_GPIO_INTMSK value ......... 0x%08X\n", (int)alt_read_word(ALT_FPGAMGR_MON_GPIO_INTMSK_ADDR));
	ALT_PRINTF("ALT_MON_GPIO_INTTYPE_LEVEL value .. 0x%08X\n", (int)alt_read_word(ALT_FPGAMGR_MON_GPIO_INTTYPE_LEVEL_ADDR));
	ALT_PRINTF("ALT_MON_GPIO_INT_POL value ........ 0x%08X\n", (int)alt_read_word(ALT_FPGAMGR_MON_GPIO_INT_POL_ADDR));
	ALT_PRINTF("ALT_MON_GPIO_INTSTAT value ........ 0x%08X\n", (int)alt_read_word(ALT_FPGAMGR_MON_GPIO_INTSTAT_ADDR));
	ALT_PRINTF("ALT_MON_GPIO_RAW_INTSTAT value .... 0x%08X\n", (int)alt_read_word(ALT_FPGAMGR_MON_GPIO_RAW_INTSTAT_ADDR));
	ALT_PRINTF("ALT_MON_GPIO_EXT_PORTA value ...... 0x%08X\n", (int)alt_read_word(ALT_FPGAMGR_MON_GPIO_EXT_PORTA_ADDR));
	ALT_PRINTF("ALT_MON_GPIO_LS_SYNC value ........ 0x%08X\n", (int)alt_read_word(ALT_FPGAMGR_MON_GPIO_LS_SYNC_ADDR));
	ALT_PRINTF("ALT_MON_GPIO_VER_ID_CODE value .... 0x%08X\n", (int)alt_read_word(ALT_FPGAMGR_MON_GPIO_VER_ID_CODE_ADDR));
	ALT_PRINTF("ALT_MON_GPIO_CFG_REG2 value ....... 0x%08X\n", (int)alt_read_word(ALT_FPGAMGR_MON_GPIO_CFG_REG2_ADDR));
	ALT_PRINTF("ALT_MON_GPIO_CFG_REG1 value ....... 0x%08X\n", (int)alt_read_word(ALT_FPGAMGR_MON_GPIO_CFG_REG1_ADDR));

	return;
}

//
// Initializes and enables the interrupt controller.
//
ALT_STATUS_CODE socfpga_int_start(void)
{
    ALT_STATUS_CODE status = ALT_E_SUCCESS;

    //
    // Initialize the global and CPU interrupt items
    //

    if (status == ALT_E_SUCCESS)
    {
        status = alt_int_global_init();
        if (status != ALT_E_SUCCESS)
        {
            PRINT_WITH_GLOBALTMR("ERROR: alt_int_global_init() failed, %d.\n", (int)status);
        }
    }

    if (status == ALT_E_SUCCESS)
    {
        status = alt_int_cpu_init();
        if (status != ALT_E_SUCCESS)
        {
            PRINT_WITH_GLOBALTMR("ERROR: alt_int_cpu_init() failed, %d.\n", (int)status);
        }
    }

    //
    // Enable the CPU and global interrupt
    //

    if (status == ALT_E_SUCCESS)
    {
        status = alt_int_cpu_enable();
        if (status != ALT_E_SUCCESS)
        {
            PRINT_WITH_GLOBALTMR("ERROR: alt_int_cpu_enable() failed, %d.\n", (int)status);
        }
    }

    if (status == ALT_E_SUCCESS)
    {
        status = alt_int_global_enable();
        if (status != ALT_E_SUCCESS)
        {
            PRINT_WITH_GLOBALTMR("ERROR: alt_int_global_enable() failed, %d.\n", (int)status);
        }
    }

    return status;
}

void socfpga_int_stop(void)
{
    //
    // Disable the global, CPU, and distributor interrupt
    //

    if (alt_int_global_disable() != ALT_E_SUCCESS)
    {
        PRINT_WITH_GLOBALTMR("WARN: alt_int_global_disable() return non-SUCCESS.");
    }

    if (alt_int_cpu_disable() != ALT_E_SUCCESS)
    {
        PRINT_WITH_GLOBALTMR("WARN: alt_int_cpu_disable() return non-SUCCESS.");
    }

    //
    // Uninitialize the CPU and global data structures.
    //

    if (alt_int_cpu_uninit() != ALT_E_SUCCESS)
    {
        PRINT_WITH_GLOBALTMR("WARN: alt_int_cpu_uninit() return non-SUCCESS.\n");
    }

    if (alt_int_global_uninit() != ALT_E_SUCCESS)
    {
        PRINT_WITH_GLOBALTMR("WARN: alt_int_global_uninit() return non-SUCCESS.\n");
    }
}

int main(int argc, char** argv)
{
    ALT_PRINTF("\r\nUser Application Start!\r\n");

    // This example only works on the Cyclone 5.
    if (!alt_hps_detect_is_cyclone5())
    {
        ALT_PRINTF("ERROR: FPGA example is specific to Cyclone 5 SoCFPGA.\n");
        ALT_PRINTF(" -> Please visit http://www.altera.com/ for more SoCFPGA examples.\n");
        return 1;
    }

    ALT_STATUS_CODE status = ALT_E_SUCCESS;
    ALT_DMA_CHANNEL_t channel = ALT_DMA_CHANNEL_0;
	ALT_STATUS_CODE result_code;

    //
    // Start the interrupt system
    //
    if (status == ALT_E_SUCCESS)
    {
        status = socfpga_int_start();
    }

	/* Usage: ALT_STATUS_CODE alt_fpga_interface_is_enabled  ( ALT_FPGA_INTERFACE_t  intfc ); 	*/
	sample_systemmanager_print_is_enable();

    /* Usage: ALT_STATUS_CODE alt_fpga_man_irq_disable(ALT_FPGA_MON_STATUS_t mon_stat_mask);	 */
	PRINT_WITH_GLOBALTMR("-- HWLib test : alt_fpga_man_irq_disable() --\n");
	result_code = alt_fpga_man_irq_disable(ALT_FPGA_MONMASK_ALL);
	if(result_code != ALT_E_SUCCESS)
	{
		ALT_PRINTF("ERROR!!: alt_fpga_man_irq_disable() result=%d\n", (int)result_code);
	} else {
		ALT_PRINTF("SUCCESS!!\n");
	}

	sample_fpgamanager_setting_gic();

	// Initialize Data For Interrupt Detect Info.
	isrlog_lastintstat = alt_read_word(ALT_FPGAMGR_MON_GPIO_INTSTAT_ADDR);
	for(int i=0; i<12; i++){
		isrlog_FPGA_MAN_IRQ_fallEdge[i].flag = false;
		isrlog_FPGA_MAN_IRQ_riseEdge[i].flag = false;
	}

    /* Usage: ALT_STATUS_CODE alt_fpga_man_irq_enable(ALT_FPGA_MON_STATUS_t mon_stat_mask);	 */
	PRINT_WITH_GLOBALTMR("-- HWLib test : alt_fpga_man_irq_enable() --\n");
	result_code = alt_fpga_man_irq_enable(ALT_FPGA_MONMASK_ALL);
	if(result_code != ALT_E_SUCCESS)
	{
		ALT_PRINTF("ERROR!!: alt_fpga_man_irq_enable() result=%d\n", (int)result_code);
	} else {
		ALT_PRINTF("SUCCESS!!\n");
	}

    util_time_init();

	sample_fpgamanager_print_fpgamgrregs_mon();
    util_time_record_start_point(MP_ALL);

    if (status == ALT_E_SUCCESS)
    {
        util_time_record_start_point(MP_SETUP_DMA_ALL);
        status = socfpga_dma_setup(&channel);
        util_time_record_end_point(MP_SETUP_DMA_ALL);
    }

    if (status == ALT_E_SUCCESS)
    {
        // This is the symbol name for the SOF file contents linked in.
        extern char _binary_soc_system_dc_rbf_start;
        extern char _binary_soc_system_dc_rbf_end;

        // Use the above symbols to extract the FPGA image information.
        const char *   fpga_image      = &_binary_soc_system_dc_rbf_start;
        const uint32_t fpga_image_size = &_binary_soc_system_dc_rbf_end - &_binary_soc_system_dc_rbf_start;

        // Trace the FPGA image information.
        PRINT_WITH_GLOBALTMR("INFO: FPGA Image binary at %p.\n", fpga_image);
        PRINT_WITH_GLOBALTMR("INFO: FPGA Image size is %" PRIu32 " bytes.\n", fpga_image_size);

        util_time_record_start_point(MP_SETUP_FPGA_ALL);
        status = socfpga_fpga_setup_dma(fpga_image, fpga_image_size, channel);
        util_time_record_end_point(MP_SETUP_FPGA_ALL);
    }

    if (status == ALT_E_SUCCESS)
    {
        util_time_record_start_point(MP_SETUP_BRIDGE_ALL);
        status = socfpga_bridge_setup(ALT_BRIDGE_LWH2F);
        util_time_record_end_point(MP_SETUP_BRIDGE_ALL);
    }

    /////

    if (status == ALT_E_SUCCESS)
    {
        util_time_record_start_point(MP_DEMOSTRATION_IO_ACROSS_BRIDGE_ALL);
        status = socfpga_bridge_io();
        util_time_record_end_point(MP_DEMOSTRATION_IO_ACROSS_BRIDGE_ALL);
    }

    /////

    /* Usage: bool alt_fpga_control_is_enabled(void);	 */
	PRINT_WITH_GLOBALTMR("-- HWLib test : alt_fpga_control_is_enabled() [result=%d] --\n", (int)alt_fpga_control_is_enabled());
	/* Usage: uint32_t alt_fpga_mon_status_get(void);	 */
	PRINT_WITH_GLOBALTMR("-- HWLib test : alt_fpga_mon_status_get() [result=0x%08X] --\n", (int)alt_fpga_mon_status_get());

    /* Usage: uint32_t alt_fpga_gpi_read(uint32_t mask);	 */
	PRINT_WITH_GLOBALTMR("-- HWLib test : alt_fpga_gpi_read() [result=0x%08X] --\n", (int)alt_fpga_gpi_read(0xFFFFFFFF));
    /* Usage: ALT_STATUS_CODE alt_fpga_gpo_write(uint32_t mask, uint32_t value);	 */
	PRINT_WITH_GLOBALTMR("-- HWLib test : alt_fpga_gpo_write() --\n");
	result_code = alt_fpga_gpo_write(0xFFFFFFFF, 0xFFFFFFFF);
	if(result_code != ALT_E_SUCCESS)
	{
		ALT_PRINTF("ERROR!!: alt_fgpa_gpo_write() result=%d\n", (int)result_code);
	} else {
		ALT_PRINTF("SUCCESS!!\n");
	}

    util_time_record_start_point(MP_CLEAN_BRIDGE_ALL);
    socfpga_bridge_cleanup(ALT_BRIDGE_LWH2F);
    util_time_record_end_point(MP_CLEAN_BRIDGE_ALL);

    util_time_record_start_point(MP_CLEAN_FPGA_ALL);
    socfpga_fpga_cleanup();
    util_time_record_end_point(MP_CLEAN_FPGA_ALL);
    util_time_record_start_point(MP_CLEAN_DMA_ALL);
    socfpga_dma_cleanup(channel);
    PRINT_WITH_GLOBALTMR("\n");
    util_time_record_end_point(MP_CLEAN_DMA_ALL);
    

    util_time_record_end_point(MP_ALL);
	sample_fpgamanager_print_fpgamgrregs_mon();

    ALT_PRINTF("\n");

    PRINT_WITH_GLOBALTMR("-- List of the measurement points and the index value. --\n");
    for(int i=0; i<MP_MAX; i++)	ALT_PRINTF("[TIME#%2d]..%s\n", i, MeasPointMessage[i]);
    ALT_PRINTF("\n");

    util_time_print_result_all(enumUTIL_TIME_TARGET_by_BOTH);

    //
    // Stop the interrupt system
    //
    socfpga_int_stop();

	// Initialize Data For Interrupt Detect Info.
    ALT_PRINTF("======== FPGA_MAN_IRQ Detect Log ========\n");
    for(int i=0; i<12; i++){
    	sample_isrloginfo_t*	isrlog_p;

    	switch(i){
    	case 0:
    		ALT_PRINTF("-- ALT_FPGA_MON_nSTATUS --\n");
    		break;
    	case 1:
			ALT_PRINTF("-- ALT_FPGA_MON_CONF_DONE --\n");
    		break;
    	case 2:
			ALT_PRINTF("-- ALT_FPGA_MON_INIT_DONE --\n");
    		break;
    	case 3:
			ALT_PRINTF("-- ALT_FPGA_MON_CRC_ERROR --\n");
    		break;
    	case 4:
			ALT_PRINTF("-- ALT_FPGA_MON_CVP_CONF_DONE --\n");
    		break;
    	case 5:
			ALT_PRINTF("-- ALT_FPGA_MON_PR_READY --\n");
    		break;
    	case 6:
			ALT_PRINTF("-- ALT_FPGA_MON_PR_ERROR --\n");
    		break;
    	case 7:
			ALT_PRINTF("-- ALT_FPGA_MON_PR_DONE --\n");
    		break;
    	case 8:
			ALT_PRINTF("-- ALT_FPGA_MON_nCONFIG_PIN --\n");
    		break;
    	case 9:
			ALT_PRINTF("-- ALT_FPGA_MON_nSTATUS_PIN --\n");
    		break;
    	case 10:
			ALT_PRINTF("-- ALT_FPGA_MON_CONF_DONE_PIN --\n");
    		break;
    	default:	/* case 11: */
    		ALT_PRINTF("-- ALT_FPGA_MON_FPGA_POWER_ON --\n");
    		break;
    	}
    	ALT_PRINTF("raising-Edge: ");
    	isrlog_p = &isrlog_FPGA_MAN_IRQ_riseEdge[i];
    	if(isrlog_p->flag){
    		ALT_PRINTF("[0x%08X]detected!! EXT_PORTA=0x%08X, INTSTAT=0x%08X\n",
    				(int)isrlog_p->globaltmrlow32, (int)isrlog_p->opt1, (int)isrlog_p->opt2);
    	} else {
    		ALT_PRINTF("non-detect\n");
    	}
    	ALT_PRINTF("falling-Edge: ");
    	isrlog_p = &isrlog_FPGA_MAN_IRQ_fallEdge[i];
    	if(isrlog_p->flag){
    		ALT_PRINTF("[0x%08X]detected!! EXT_PORTA=0x%08X, INTSTAT=0x%08X\n",
    				(int)isrlog_p->globaltmrlow32, (int)isrlog_p->opt1, (int)isrlog_p->opt2);
    	} else {
    		ALT_PRINTF("non-detect\n");
    	}
	}
    ALT_PRINTF("\n");

    if (status == ALT_E_SUCCESS)
    {
        PRINT_WITH_GLOBALTMR("RESULT: Example completed successfully.\n");
        return 0;
    }
    else
    {
        PRINT_WITH_GLOBALTMR("RESULT: Some failures detected.\n");
        return 1;
    }
}
