﻿/********************************************************************************//*!
 * @file  sample_adrspacemanager.c
 * @brief  To use HWLib for MMU Management API example program.
 * 
 * @details  Run the API category below.
 *           - MMU Management API
 *               - MMU Management
 *               - MMU Virtual Address Space Creation
 * 
 * @note  nothing. 
 * 
 * @date <b> History: \<Date\> \<Rev\> \<Keyword\> \<Details\> </b>
 * @date 2014/03/03  ---  #13288  新規作成. 
 * @date 2014/08/18  ---  -----  v14.0化 (誤植訂正、操作説明の表示を追加). 
 * @date 2015/06/25  ---  #20263  v15.0対応/文字コード変更(UTF-8) 他. 
 * 
 * @attention 
 * Copyright (C) 2013-2014 ALTIMA Corp./ELSENA,Inc. All Rights Reserved.\n
 *   This software is licensed "AS IS". 
 *   Please perform use of this software by a user's own responsibility and expense.
 *   It cannot guarantee in the maker side about the damage which occurred by the ab-
 *   ility not to use or use this software, and all damage that occurred secondarily.
 **//*******************************************************************************/

/***********************************************************************************
 *  includes 
 ***********************************************************************************/
#include <stdio.h>
#include <string.h>
#include "hwlib.h"
#include "alt_generalpurpose_io.h"
#include "alt_globaltmr.h"
#include "alt_clock_manager.h"
#include "alt_interrupt.h"
#include "alt_address_space.h"
#include "alt_mmu.h"

/***********************************************************************************
 *  externs 
 ***********************************************************************************/

/***********************************************************************************
 *  proto types 
 ***********************************************************************************/
static void sample_adrspcmng_setting_remap(void);
static void sample_adrspcmng_setting_l2addrfilter(uint32_t addr_filt_start, uint32_t addr_filt_end);
static void sample_adrspcmng_print_ACPIDMapper(void);
void* sample_mmu_tlb_alloc(const size_t size, void * context);
void sample_mmu_init_MMU(void);
void sample_mmu_test_ttb_desc_set_APIs(void);
void sample_mmu_test_TLB_maintenance_operations_APIs(void);
void sample_mmu_print_registers(void);
void sample_mmu_test_register_setget_APIs(void);
void sample_test_init(void);
void sample_test_uninit(void);
void sample_test_print_usage(void);
void sample_test_main(void);
// prototype (Common Functions) //
static void sample_gpio_config_helioboard(void);
static void sample_dmac_setting_testdata(uint32_t* start, uint32_t* endnext);

/***********************************************************************************
 *  definitions (define, enum, typedef, etc..) 
 ***********************************************************************************/
/** memo: port asign ***************/
/* Signal Name <--> HPS Port Name  */
/* USER_DIPSW_HPS0 <--> HPS_GPI13  */
/* USER_DIPSW_HPS1 <--> HPS_GPI12  */
/* USER_DIPSW_HPS2 <--> HPS_GPI11  */
/* USER_DIPSW_HPS3 <--> HPS_GPI10  */
/* USER_PB_HPS0 <--> HPS_GPI9      */
/* USER_PB_HPS1 <--> HPS_GPI8      */
/* USER_PB_HPS2 <--> HPS_GPI7      */
/* USER_PB_HPS3 <--> HPS_GPI6      */
/* USER_HPS_GPIO61 <--> HPS_GPIO61 */
/***********************************/
#define ALT_GPIO_PORTBIT_DIPS1		(ALT_GPIO_BIT26)
#define ALT_GPIO_PORTBIT_DIPS2		(ALT_GPIO_BIT25)
#define ALT_GPIO_PORTBIT_DIPS3		(ALT_GPIO_BIT24)
#define ALT_GPIO_PORTBIT_DIPS4		(ALT_GPIO_BIT23)
#define ALT_GPIO_PORTBIT_DIPSWALL	((ALT_GPIO_BIT26)|(ALT_GPIO_BIT25)|(ALT_GPIO_BIT24)|(ALT_GPIO_BIT23))
#define ALT_GPIO_LSBBIT_DIPSWS		(23)
#define ALT_GPIO_PORTBIT_PUSH2		(ALT_GPIO_BIT22)
#define ALT_GPIO_PORTBIT_PUSH0		(ALT_GPIO_BIT21)
#define ALT_GPIO_PORTBIT_PUSH3		(ALT_GPIO_BIT20)
#define ALT_GPIO_PORTBIT_PUSH1		(ALT_GPIO_BIT19)
#define ALT_GPIO_PORTBIT_PUSHSWALL	((ALT_GPIO_BIT22)|(ALT_GPIO_BIT21)|(ALT_GPIO_BIT20)|(ALT_GPIO_BIT19))
#define ALT_GPIO_PORTBIT_SWALL		((ALT_GPIO_PORTBIT_DIPSWALL)|(ALT_GPIO_PORTBIT_PUSHSWALL))

/* Basic definition*/
#define	_KB		1024UL
#define	_MB		(1024UL*1024UL)
#define	_GB		(1024UL*1024UL*1024UL)

/* MMU table buffer size */
#define	MMU_TLB_HEAP_SIZE	( 32 * _KB )

/* TLB memory should be aligned 16KB */
#define	MMU_TLB_ALIGNMENT		(16*_KB)

/******** Definition Memory Region ********/
#define MAX_MEMORY_REGION_NUM	16

/*! SDRAM .. PA:0x00000000~0x003FFFFF <--> VA:0x00000000~0x003FFFFF */
#define	SDRAM_PHYSICAL_ADDR			0x00000000
#define	SDRAM_VIRTUAL_ADDR			SDRAM_PHYSICAL_ADDR
#define	SDRAM_SIZE					( 4 * _MB )

/*! SDRAM2 .. PA:0x00800000~0x00BFFFFF <--> VA:0x00400000~0x007FFFFF */
#define	SDRAM2_PHYSICAL_ADDR		0x00800000
#define	SDRAM2_VIRTUAL_ADDR			0x00400000
#define	SDRAM2_SIZE					( 4 * _MB )

/*! SDRAM3 .. PA:0x02000000~0x03FFFFFF <--> VA:0x02000000~0x03FFFFFF */
#define	SDRAM3_PHYSICAL_ADDR		0x02000000
#define	SDRAM3_VIRTUAL_ADDR			SDRAM3_PHYSICAL_ADDR
#define	SDRAM3_SIZE					((1024 * _MB) - (32 * _MB))

/*! HPS2FPGA .. PA:0xC000000~0xFBFFFFFF <--> VA:0xC0000000~0xFBFFFFFF */
#define	HPS2FPGA_PHYSICAL_ADDR		0xC0000000
#define	HPS2FPGA_VIRTUAL_ADDR		HPS2FPGA_PHYSICAL_ADDR
#define	HPS2FPGA_SIZE				( 960 * _MB )

/*! LW_HPS2FPGA .. PA:0xFF200000~0xFF3FFFFF <--> VA:0xFF200000~0xFF3FFFFF */
#define	LW_HPS2FPGA_PHYSICAL_ADDR	0xFF200000
#define	LW_HPS2FPGA_VIRTUAL_ADDR	LW_HPS2FPGA_PHYSICAL_ADDR
#define	LW_HPS2FPGA_SIZE			( 2 * _MB )

/*! PERIPHRALS .. PA:0xFF400000~0xFFEFFFFF <--> VA:0xFF400000~0xFFEFFFFF */
#define	PERIPHRALS_PHYSICAL_ADDR	0xFF400000
#define	PERIPHRALS_VIRTUAL_ADDR		PERIPHRALS_PHYSICAL_ADDR
#define	PERIPHRALS_SIZE				( 11 * _MB )

/*! BOOTROM .. PA:0xFFFD0000~0xFFFDFFFF <--> VA:0xFFFD0000~0xFFFDFFFF */
#define	BOOTROM_PHYSICAL_ADDR		0xFFFD0000
#define	BOOTROM_VIRTUAL_ADDR		BOOTROM_PHYSICAL_ADDR
#define	BOOTROM_SIZE				( 64 *_KB )

/*! MPUSCU .. PA:0xFFFEC000~0xFFFEDFFF <--> VA:0xFFFEC000~0xFFFEDFFF */
#define	MPUSCU_PHYSICAL_ADDR		0xFFFEC000
#define	MPUSCU_VIRTUAL_ADDR			MPUSCU_PHYSICAL_ADDR
#define	MPUSCU_SIZE					( 8 * _KB)

/*! MPUL2 .. PA:0xFFFEF000~0xFFFEFFFF <--> VA:0xFFFEF000~0xFFFEFFFF */
#define	MPUL2_PHYSICAL_ADDR			0xFFFEF000
#define	MPUL2_VIRTUAL_ADDR			MPUL2_PHYSICAL_ADDR
#define	MPUL2_SIZE					( 4 * _KB)

/*! ONCHIPRAM .. PA:0xFFFF0000~0xFFFFFFFF <--> VA:0xFFFF0000~0xFFFFFFFF */
#define	RAM_PHYSICAL_ADDR			0xFFFF0000
#define	RAM_VIRTUAL_ADDR			RAM_PHYSICAL_ADDR
#define	RAM_SIZE					( 64 *_KB )

/***********************************************************************************
 *  variables
 ***********************************************************************************/
//typedef struct ALT_MMU_MEM_REGION_s
//{
//  void *                 va;        /*!< The beginning virtual address for the memory
//                                     *   region. The address must be aligned to one of 4KiB,
//                                     *   64KiB, 1MiB, or 16MiB boundaries.
//                                     */
//  void *                 pa;        /*!< The beginning physical address mapping for the
//                                     *   virtual address of the memory region. The address
//                                     *   must be aligned to one of 4KiB, 64KiB, 1MiB, or 16MiB
//                                     *   boundaries.
//                                     */
//  uint32_t               size;      /*!< The size of the memory region in bytes. The size
//                                     *   must be a multiple of 4KiB, 64KiB, 1MiB, or 16MiB
//                                     *   sizes.
//                                     */
//  ALT_MMU_AP_t           access;    /*!< The access permissions for the memory region. */
//  ALT_MMU_ATTR_t         attributes;/*!< The memory region attributes. These attributes
//                                     *   determine the memory type (ordering), cache
//                                     *   policy, and as a possible side effect, the
//                                     *   shareablity of the memory region.
//                                     */
//  ALT_MMU_TTB_S_t        shareable; /*!< The shareability of the memory region. */
//  ALT_MMU_TTB_XN_t       execute;   /*!< Whether instructions can be executed from this
//                                     *   memory region.
//                                     */
//  ALT_MMU_TTB_NS_t       security;  /*!< Controls whether address translations made from
//                                     *   the secure state translate physical address in
//                                     *   the secure or non-secure address map.
//                                     */
//} ALT_MMU_MEM_REGION_t;
int	ValidMemoryRegion = 10;		/*!< The number of valid descriptors. */
ALT_MMU_MEM_REGION_t MemoryRegion[MAX_MEMORY_REGION_NUM] = {
	/* [0] SDRAM	*/
	{ (void*)SDRAM_VIRTUAL_ADDR, (void*)SDRAM_PHYSICAL_ADDR, SDRAM_SIZE,	/* va, pa, size, */
	  ALT_MMU_AP_FULL_ACCESS,		/* access, */
	  ALT_MMU_ATTR_WBA,				/* attributes, */
	  ALT_MMU_TTB_S_NON_SHAREABLE,	/* shareable, *//* This value must be match with TTBR0.S if TLB on this area */
	  ALT_MMU_TTB_XN_DISABLE,		/* execute, */
	  ALT_MMU_TTB_NS_SECURE			/* security */
	},
	/* [1] SDRAM2	*/
	{ (void*)SDRAM2_VIRTUAL_ADDR, (void*)SDRAM2_PHYSICAL_ADDR, SDRAM2_SIZE,	/* va, pa, size, */
	  ALT_MMU_AP_FULL_ACCESS,		/* access, */
	  ALT_MMU_ATTR_WBA,				/* attributes, */
	  ALT_MMU_TTB_S_NON_SHAREABLE,	/* shareable, *//* This value must be match with TTBR0.S if TLB on this area */
	  ALT_MMU_TTB_XN_DISABLE,		/* execute, */
	  ALT_MMU_TTB_NS_SECURE			/* security */
	},
	/* [2] SDRAM3 (include .text)	*/
	{ (void*)SDRAM3_VIRTUAL_ADDR, (void*)SDRAM3_PHYSICAL_ADDR, SDRAM3_SIZE,	/* va, pa, size, */
	  ALT_MMU_AP_FULL_ACCESS,		/* access, */
	  ALT_MMU_ATTR_WBA,				/* attributes, */
	  ALT_MMU_TTB_S_NON_SHAREABLE,	/* shareable, *//* This value must be match with TTBR0.S if TLB on this area */
	  ALT_MMU_TTB_XN_DISABLE,		/* execute, */
	  ALT_MMU_TTB_NS_SECURE			/* security */
	},
	/* [3] HPS2FPGA	*/
	{ (void*)HPS2FPGA_VIRTUAL_ADDR, (void*)HPS2FPGA_PHYSICAL_ADDR, HPS2FPGA_SIZE,	/* va, pa, size, */
	  ALT_MMU_AP_FULL_ACCESS,		/* access, */
	  ALT_MMU_ATTR_DEVICE,			/* attributes, */
	  ALT_MMU_TTB_S_SHAREABLE,		/* shareable, */
	  ALT_MMU_TTB_XN_ENABLE,		/* execute, */
	  ALT_MMU_TTB_NS_SECURE			/* security */
	},
	/* [4] LW_HPS2FPGA	*/
	{ (void*)LW_HPS2FPGA_VIRTUAL_ADDR, (void*)LW_HPS2FPGA_PHYSICAL_ADDR, LW_HPS2FPGA_SIZE,	/* va, pa, size, */
	  ALT_MMU_AP_FULL_ACCESS,		/* access, */
	  ALT_MMU_ATTR_DEVICE,			/* attributes, */
	  ALT_MMU_TTB_S_SHAREABLE,		/* shareable, */
	  ALT_MMU_TTB_XN_ENABLE,		/* execute, */
	  ALT_MMU_TTB_NS_SECURE			/* security */
	},
	/* [5] PERIPHRALS	*/
	{ (void*)PERIPHRALS_VIRTUAL_ADDR, (void*)PERIPHRALS_PHYSICAL_ADDR, PERIPHRALS_SIZE,	/* va, pa, size, */
	  ALT_MMU_AP_FULL_ACCESS,		/* access, */
	  ALT_MMU_ATTR_STRONG,			/* attributes, */
	  ALT_MMU_TTB_S_SHAREABLE,		/* shareable, */
	  ALT_MMU_TTB_XN_ENABLE,		/* execute, */
	  ALT_MMU_TTB_NS_SECURE			/* security */
	},
	/* [6] BOOTROM	*/
	{ (void*)BOOTROM_VIRTUAL_ADDR, (void*)BOOTROM_PHYSICAL_ADDR, BOOTROM_SIZE,	/* va, pa, size, */
	  ALT_MMU_AP_READ_ONLY,			/* access, */
	  ALT_MMU_ATTR_WBA,				/* attributes, */
	  ALT_MMU_TTB_S_SHAREABLE,		/* shareable, */
	  ALT_MMU_TTB_XN_DISABLE,		/* execute, */
	  ALT_MMU_TTB_NS_SECURE			/* security */
	},
	/* [7] MPU_SCU	*/
	{ (void*)MPUSCU_VIRTUAL_ADDR, (void*)MPUSCU_PHYSICAL_ADDR, MPUSCU_SIZE,	/* va, pa, size, */
	  ALT_MMU_AP_FULL_ACCESS,		/* access, */
	  ALT_MMU_ATTR_STRONG,			/* attributes, */
	  ALT_MMU_TTB_S_NON_SHAREABLE,	/* shareable, */
	  ALT_MMU_TTB_XN_ENABLE,		/* execute, */
	  ALT_MMU_TTB_NS_SECURE			/* security */
	},
	/* [8] MPU_L2	*/
	{ (void*)MPUL2_VIRTUAL_ADDR, (void*)MPUL2_PHYSICAL_ADDR, MPUL2_SIZE,	/* va, pa, size, */
	  ALT_MMU_AP_FULL_ACCESS,		/* access, */
	  ALT_MMU_ATTR_STRONG,			/* attributes, */
	  ALT_MMU_TTB_S_NON_SHAREABLE,	/* shareable, */
	  ALT_MMU_TTB_XN_ENABLE,		/* execute, */
	  ALT_MMU_TTB_NS_SECURE			/* security */
	},
	/* [9] RAM	*/
	{ (void*)RAM_VIRTUAL_ADDR, (void*)RAM_PHYSICAL_ADDR, RAM_SIZE,	/* va, pa, size, */
	  ALT_MMU_AP_FULL_ACCESS,		/* access, */
	  ALT_MMU_ATTR_WBA,				/* attributes, */
	  ALT_MMU_TTB_S_SHAREABLE,		/* shareable, */
	  ALT_MMU_TTB_XN_DISABLE,		/* execute, */
	  ALT_MMU_TTB_NS_SECURE			/* security */
	},
	/* [10] Undefined */
	{ 0, 0, 0,	/* va, pa, size, */
	  0,		/* access, */
	  0,		/* attributes, */
	  0,		/* shareable, */
	  0,		/* execute, */
	  0			/* security */
	},
	/* [11] Undefined */
	{ 0, 0, 0,	/* va, pa, size, */
	  0,		/* access, */
	  0,		/* attributes, */
	  0,		/* shareable, */
	  0,		/* execute, */
	  0			/* security */
	},
	/* [12] Undefined */
	{ 0, 0, 0,	/* va, pa, size, */
	  0,		/* access, */
	  0,		/* attributes, */
	  0,		/* shareable, */
	  0,		/* execute, */
	  0			/* security */
	},
	/* [13] Undefined */
	{ 0, 0, 0,	/* va, pa, size, */
	  0,		/* access, */
	  0,		/* attributes, */
	  0,		/* shareable, */
	  0,		/* execute, */
	  0			/* security */
	},
	/* [14] Undefined */
	{ 0, 0, 0,	/* va, pa, size, */
	  0,		/* access, */
	  0,		/* attributes, */
	  0,		/* shareable, */
	  0,		/* execute, */
	  0			/* security */
	},
	/* [15] Undefined */
	{ 0, 0, 0,	/* va, pa, size, */
	  0,		/* access, */
	  0,		/* attributes, */
	  0,		/* shareable, */
	  0,		/* execute, */
	  0			/* security */
	}
};

static uint32_t mmu_tlb_heap[MMU_TLB_HEAP_SIZE] __attribute__ ((aligned MMU_TLB_ALIGNMENT)) ;
uint32_t *mmu_tlb_l1 = NULL;

const char* PageAddressString[4] = {
	"Page 0 - MPU address range 0x00000000 - 0x3FFFFFFF",
	"Page 1 - MPU address range 0x40000000 - 0x7FFFFFFF",
	"Page 2 - MPU address range 0x80000000 - 0xBFFFFFFF",
	"Page 3 - MPU address range 0xC0000000 - 0xFFFFFFFF"
};

/***********************************************************************************
 *  functions 
 ***********************************************************************************/

/********************************************************************************//*!
 * @fn  static void sample_adrspcmng_setting_remap(void)
 * @brief  Setting for the Address Space Remap Register.
 *
 * @details  Run the following APIs.
 *           - alt_addr_space_remap
 *           - alt_mpu_addr_space_remap_0_to_sdram
 * @param[in,out] void  none
 * @retval void  none
 *
 * @attention  nothing.
 * @pre        nothing.
 * @post       nothing.
 * @note       nothing.
 *
 * @date <b> History: \<Date\> \<Rev\> \<Keyword\> \<Details\> </b>
 * @date 2014/02/13  ---  #13144  新規作成.
 **//*******************************************************************************/
static void sample_adrspcmng_setting_remap(void)
{
	ALT_STATUS_CODE result_code;
	// Remap Setting Value 
	ALT_ADDR_SPACE_MPU_ATTR_t			mpu_attr = ALT_ADDR_SPACE_MPU_ZERO_AT_OCRAM;
	ALT_ADDR_SPACE_NONMPU_ATTR_t		nonmpu_attr = ALT_ADDR_SPACE_NONMPU_ZERO_AT_SDRAM;
	ALT_ADDR_SPACE_H2F_BRIDGE_ATTR_t	h2f_attr = ALT_ADDR_SPACE_H2F_INACCESSIBLE;
	ALT_ADDR_SPACE_LWH2F_BRIDGE_ATTR_t	lwh2f_attr = ALT_ADDR_SPACE_LWH2F_INACCESSIBLE;
	
	/* -- Address Space Mapping Control -- */
	/* Usage:	ALT_STATUS_CODE  alt_addr_space_remap (ALT_ADDR_SPACE_MPU_ATTR_t mpu_attr, ALT_ADDR_SPACE_NONMPU_ATTR_t nonmpu_attr, ALT_ADDR_SPACE_H2F_BRIDGE_ATTR_t h2f_attr, ALT_ADDR_SPACE_LWH2F_BRIDGE_ATTR_t lwh2f_attr);	*/
	/* Usage:	ALT_STATUS_CODE  alt_mpu_addr_space_remap_0_to_sdram (void);	*/
	printf("==== HWLib test : Execute the Address Space Remap Setting ====\n");
	
	result_code = alt_addr_space_remap(mpu_attr, nonmpu_attr, h2f_attr, lwh2f_attr);
	if(result_code != ALT_E_SUCCESS){
		printf("ERROR!!: ");
	}
	printf("alt_addr_space_remap() .... result=%d\n", (int)result_code);
	
	result_code = alt_mpu_addr_space_remap_0_to_sdram();
	if(result_code != ALT_E_SUCCESS){
		printf("ERROR!!: ");
	}
	printf("alt_mpu_addr_space_remap_0_to_sdram() .... result=%d\n", (int)result_code);
	
	printf("\n");
	
	return;
}

/********************************************************************************//*!
 * @fn  static void sample_adrspcmng_setting_l2addrfilter(uint32_t addr_filt_start, uint32_t addr_filt_end)
 * @brief  Setting for the L2 Cache Address Filter.
 *
 * @details  Run the following APIs.
 *           - alt_l2_addr_filter_cfg_get
 *           - alt_l2_addr_filter_cfg_set
 * @param[in] addr_filt_start  Filter Start Address.
 * @param[in] addr_filt_end  Filter End Address.
 * @retval void  none
 *
 * @attention  nothing.
 * @pre        nothing.
 * @post       nothing.
 * @note       nothing.
 *
 * @date <b> History: \<Date\> \<Rev\> \<Keyword\> \<Details\> </b>
 * @date 2014/02/13  ---  #13144  新規作成.
 **//*******************************************************************************/
static void sample_adrspcmng_setting_l2addrfilter(uint32_t addr_filt_start, uint32_t addr_filt_end)
{
	ALT_STATUS_CODE result_code;
	uint32_t	before_filt_start;
	uint32_t	before_filt_end;
	
	/* -- L2 Cache Address Filter -- */
	/* Usage:	ALT_STATUS_CODE  alt_l2_addr_filter_cfg_get (uint32_t *addr_filt_start, uint32_t *addr_filt_end);	*/
	/* Usage:	ALT_STATUS_CODE  alt_l2_addr_filter_cfg_set (uint32_t addr_filt_start, uint32_t addr_filt_end);	*/
	printf("==== HWLib test : Execute the L2 Cache Address Filter Setting ====\n");
	result_code = alt_l2_addr_filter_cfg_get(&before_filt_start, &before_filt_end);
	printf("<<before setting>>\n");
	if(result_code != ALT_E_SUCCESS){
		printf("ERROR!!: ");
	}
	printf("alt_l2_addr_filter_cfg_get() .... result=%d\n", (int)result_code);
	printf(" +-- addr_filt_start = 0x%08X\n", (int)before_filt_start);
	printf(" +-- addr_filt_end   = 0x%08X\n", (int)before_filt_end);
	
	result_code = alt_l2_addr_filter_cfg_set(addr_filt_start, addr_filt_end);
	printf("<<after setting>>\n");
	if(result_code != ALT_E_SUCCESS){
		printf("ERROR!!: ");
	}
	printf("alt_l2_addr_filter_cfg_set() .... result=%d\n", (int)result_code);
	printf(" +-- addr_filt_start = 0x%08X\n", (int)addr_filt_start);
	printf(" +-- addr_filt_end   = 0x%08X\n", (int)addr_filt_end);
	
	printf("\n");
	
	return;
}

/********************************************************************************//*!
 * @fn  static void sample_adrspcmng_print_ACPIDMapper(void)
 * @brief  Print Current Settings of ACP ID Mapper.
 *
 * @details  Run the following APIs.
 *           - alt_acpid_map_read_options_get
 *           - alt_acpid_map_write_options_get
 * @param[in,out] void  none
 * @retval void  none
 *
 * @attention  Target APIs are not implemented in the current version.
 * @pre        nothing.
 * @post       nothing.
 * @note       nothing.
 *
 * @date <b> History: \<Date\> \<Rev\> \<Keyword\> \<Details\> </b>
 * @date 2014/02/13  ---  #13144  新規作成.
 **//*******************************************************************************/
static void sample_adrspcmng_print_ACPIDMapper(void)
{
#if 0
	ALT_STATUS_CODE result_code;
	int i;
	bool fixed;
	uint32_t input_id;
	ALT_ACP_ID_MAP_PAGE_t page;
	uint32_t aruser;

	/* -- ACP Memory Coherence and ID Mapping -- */
	/* Usage:	ALT_STATUS_CODE  alt_acpid_map_read_options_get (const uint32_t output_id, bool *fixed, uint32_t *input_id, ALT_ACP_ID_MAP_PAGE_t *page, uint32_t *aruser);	*/
	/* Usage:	ALT_STATUS_CODE  alt_acpid_map_write_options_get (const uint32_t output_id, bool *fixed, uint32_t *input_id, ALT_ACP_ID_MAP_PAGE_t *page, uint32_t *awuser);	*/
	printf("==== HWLib test : Print Current Setting of ACP ID Mapper ====\n");
	printf("<<Read Transactions>>\n");
	for(i=0; i<=7; i++)
	{
		result_code = alt_acpid_map_read_options_get(i, &fixed, &input_id, &page, &aruser);
		if(result_code != ALT_E_SUCCESS){
			printf("ERROR!!: ");
		}
		printf("[%d] alt_acpid_map_read_options_get() .... result=%d\n", i, (int)result_code);
		printf(" +-- output_id = %d\n", i);
		printf(" +-- fixed = %s\n", ((fixed == true)?"true":"false"));
		printf(" +-- input_id = 0x%08X (%d)\n", (int)input_id, (int)input_id);
		printf(" +-- page = %s\n", PageAddressString[page]);
		printf(" +-- aruser = 0x%08X (%d)\n", (int)aruser, (int)aruser);
		printf("\n");
	}

	printf("<<Write Transactions>>\n");
	for(i=0; i<=7; i++)
	{
		result_code = alt_acpid_map_write_options_get(i, &fixed, &input_id, &page, &aruser);
		if(result_code != ALT_E_SUCCESS){
			printf("ERROR!!: ");
		}
		printf("[%d] alt_acpid_map_write_options_get() .... result=%d\n", i, (int)result_code);
		printf(" +-- output_id = %d\n", i);
		printf(" +-- fixed = %s\n", ((fixed == true)?"true":"false"));
		printf(" +-- input_id = 0x%08X (%d)\n", (int)input_id, (int)input_id);
		printf(" +-- page = %s\n", PageAddressString[page]);
		printf(" +-- aruser = 0x%08X (%d)\n", (int)aruser, (int)aruser);
		printf("\n");
	}

	printf("\n");
#endif
	return;
}

/********************************************************************************//*!
 * @fn  void* sample_mmu_tlb_alloc(const size_t size, void * context)
 * @brief  Memory allocation for 1st and 2nd translation tables.
 *
 * @details  User defined function used for allocating storage for first and second level translation tables.
 *           This is the function to specify the 4th argument of the API 'alt_mmu_va_space_create'.
 * @param[in] size  The size in bytes of the storage request. The argument will be either the value ALT_MMU_TTB1_SIZE or ALT_MMU_TTB2_SIZE.
 * @param[in] context  A user provided context for the allocator function.
 * @retval void*  A sixteen-word aligned pointer to the allocated memory or NULL if the storage request cannot be satisfied. 
 *
 * @attention  nothing.
 * @pre        nothing.
 * @post       nothing.
 * @note       nothing.
 *
 * @date <b> History: \<Date\> \<Rev\> \<Keyword\> \<Details\> </b>
 * @date 2014/03/03  ---  #13288  新規作成.
 **//*******************************************************************************/
void* sample_mmu_tlb_alloc(const size_t size, void * context)
{
	void* p = NULL;
	printf("Execute ttb_alloc Function.\n");

	/* User defined function used for allocating storage for first and second level translation tables. */

	if(size >= sizeof(mmu_tlb_heap)){
		printf("ERROR!!: mmu_tlb_alloc() 1st argument ""size"" is too large!\n");
	} else if(((uint32_t)context % MMU_TLB_ALIGNMENT) != 0){
		printf("ERROR!!: mmu_tlb_alloc() 2st argument ""context"" is not aligned!\n");
	} else {
		p = context;
	}

	return p ;
}

/********************************************************************************//*!
 * @fn  void sample_mmu_init_MMU(void)
 * @brief  Do the initial settings of the MMU.
 *
 * @details  Run the following APIs.
 *           - alt_mmu_disable
 *           - alt_mmu_va_space_create
 *           - alt_mmu_va_space_enable
 * @param[in,out] void  none
 * @retval void  none
 *
 * @attention  nothing.
 * @pre        nothing.
 * @post       nothing.
 * @note       nothing.
 *
 * @date <b> History: \<Date\> \<Rev\> \<Keyword\> \<Details\> </b>
 * @date 2014/03/03  ---  #13288  新規作成.
 **//*******************************************************************************/
void sample_mmu_init_MMU(void)
{
	ALT_STATUS_CODE result_code;

	/* disable MMU */
	alt_mmu_disable();

	/* create MMU tables */
	/* Usage:	ALT_STATUS_CODE  alt_mmu_va_space_create (uint32_t **ttb1, const ALT_MMU_MEM_REGION_t *mem_regions, const size_t num_mem_regions, alt_mmu_ttb_alloc_t ttb_alloc, void *ttb_alloc_context);	*/
	result_code = alt_mmu_va_space_create(&mmu_tlb_l1, MemoryRegion, ValidMemoryRegion, sample_mmu_tlb_alloc, (void*)mmu_tlb_heap );
	if(result_code != ALT_E_SUCCESS){
		printf("ERROR!!: ");
	}
	printf("alt_mmu_va_space_create() .... result=%d\n", (int)result_code);

	if(result_code == ALT_E_SUCCESS){
		/* enable MMU */
		/* Usage:	ALT_STATUS_CODE  alt_mmu_va_space_enable (const uint32_t *ttb1);	*/
		result_code = alt_mmu_va_space_enable(mmu_tlb_l1);
		if(result_code != ALT_E_SUCCESS){
			printf("ERROR!!: ");
		}
		printf("alt_mmu_va_space_enable() .... result=%d\n", (int)result_code);
	}

	if(result_code != ALT_E_SUCCESS){
		printf("MMU Init error\n" );
	}

	return;
}

/********************************************************************************//*!
 * @fn  void sample_mmu_test_ttb_desc_set_APIs(void)
 * @brief  Test the API for setting transration table descriptor.
 *
 * @details  Run the following APIs.
 *           - alt_mmu_ttb1_desc_set
 *           - alt_mmu_ttb2_desc_set
 * @param[in,out] void  none
 * @retval void  none
 *
 * @attention  nothing.
 * @pre        nothing.
 * @post       nothing.
 * @note       nothing.
 *
 * @date <b> History: \<Date\> \<Rev\> \<Keyword\> \<Details\> </b>
 * @date 2014/03/03  ---  #13288  新規作成.
 **//*******************************************************************************/
void sample_mmu_test_ttb_desc_set_APIs(void)
{
	ALT_STATUS_CODE result_code;
	uint32_t* discp_ttb1;
	uint32_t* discp_ttb2;
	uint32_t discv;
	uint32_t target_addr;
	int i;

	/* disable MMU */
	alt_mmu_disable() ;

	/* Usage:	ALT_STATUS_CODE  alt_mmu_ttb1_desc_set (uint32_t *ttb1, const void *va, const uint32_t desc);	*/
	/* -- For the tests to customize TTB1. -- */
	/* original : SDRAM3 .. PA:0x02000000~0x03FFFFFF <--> VA:0x02000000~0x03FFFFFF */
	/* <After customization> */
	/* PA:0x02000000~0x03FFFFFF <--> VA:0x02000000~0x03FFFFFF */
	/* PA:0x03000000~0x03FFFFFF <--> VA:0x04000000~0x04FFFFFF */
	/* VA:0x03000000~0x03FFFFFF is not allocated. */
	discp_ttb1 = (uint32_t*)alt_mmu_TTBR0_get();
	target_addr = 0x03000000;
	discv = discp_ttb1[(target_addr / (1 * _MB))];	/* Get discriptor value used for VA:0x03000000 */
	for(i = 0; i < 16; i++){
		/* VA:0x03000000~0x03FFFFFF is not allocated. */
		result_code = alt_mmu_ttb1_desc_set(discp_ttb1, (void*)(target_addr + (i * (1 * _MB))), 0);
		if(result_code != ALT_E_SUCCESS){
			printf("ERROR!!: ");
		}
		printf("alt_mmu_ttb1_desc_set(VA:0x%08X,0) .... result=%d\n", (int)target_addr, (int)result_code);
	}
	/* PA:0x03000000~0x03FFFFFF <--> VA:0x04000000~0x04FFFFFF */
	result_code = alt_mmu_ttb1_desc_set(discp_ttb1, (void*)(0x01000000 + target_addr), discv);
	if(result_code != ALT_E_SUCCESS){
		printf("ERROR!!: ");
	}
	printf("alt_mmu_ttb1_desc_set(VA:0x%08X,0x%08X) .... result=%d\n", (int)(0x01000000 + target_addr), (int)discv, (int)result_code);

	/* Usage:	ALT_STATUS_CODE  alt_mmu_ttb2_desc_set (const uint32_t *ttb1, const void *va, const uint32_t desc);	*/
	/* -- For the tests to customize TTB2. -- */
	/* original :		   PA:0xFFF00000~0xFFFCFFFF <--> VA: not allocated        */
	/*		  ONCHIPRAM .. PA:0xFFFF0000~0xFFFFFFFF <--> VA:0xFFFF0000~0xFFFFFFFF */
	/* <After customization> */
	/*		  ONCHIPRAM .. PA:0xFFFF0000~0xFFFFFFFF <--> VA:0xFFF00000~0xFFF0FFFF */
	/* 		               VA:0xFFFF0000~0xFFFFFFFF is not allocated. */
	discv = discp_ttb1[(0xFFF00000 / (1 * _MB))];
	discp_ttb2 = (uint32_t*)(discv & ALT_MMU_TTB1_PAGE_TBL_BASE_ADDR_MASK);
	discv = discp_ttb2[((0xFFFFF000 & ((1 * _MB) - 1)) / (4 * _KB))];	/* Get discriptor value used for VA:0xFFFFF000 */

	//discv &= ALT_MMU_TTB2_SMALL_PAGE_BASE_ADDR_MASK;
	//discv |= ALT_MMU_TTB2_SMALL_PAGE_BASE_ADDR_SET(0xFFFFF000);	/* Set discriptor value to PA:0xFFFFF000 */

	/* VA:0xFFFF0000~0xFFFFFFFF is not allocated. */
	for(i = 0; i < 16; i++){
		result_code = alt_mmu_ttb2_desc_set(discp_ttb1, (void*)(0xFFFF0000 + (i * 0x1000)), 0);
		if(result_code != ALT_E_SUCCESS){
			printf("ERROR!!: ");
		}
		printf("alt_mmu_ttb2_desc_set(VA:0x%08X,0) .... result=%d\n", (int)(0xFFFF0000 + (i * 0x1000)), (int)result_code);
	}
	/* PA:0xFFFF0000~0xFFFFFFFF <--> VA:0xFFF00000~0xFFF0FFFF */
	result_code = alt_mmu_ttb2_desc_set(discp_ttb1, (void*)0xFFF00000, discv);
	if(result_code != ALT_E_SUCCESS){
		printf("ERROR!!: ");
	}
	printf("alt_mmu_ttb2_desc_set(VA:0xFFF00000,0x%08X) .... result=%d\n", (int)discv, (int)result_code);

	/* enable MMU */
	alt_mmu_enable() ;

	return;
}

/********************************************************************************//*!
 * @fn  void sample_mmu_test_TLB_maintenance_operations_APIs(void)
 * @brief  Test the API for TLB maintenance operation.
 *
 * @details  Run the following APIs.
 *           - alt_mmu_tlb_invalidate
 *           - alt_mmu_tlb_invalidate_is
 * @param[in,out] void  none
 * @retval void  none
 *
 * @attention  nothing.
 * @pre        nothing.
 * @post       nothing.
 * @note       nothing.
 *
 * @date <b> History: \<Date\> \<Rev\> \<Keyword\> \<Details\> </b>
 * @date 2014/03/03  ---  #13288  新規作成.
 **//*******************************************************************************/
void sample_mmu_test_TLB_maintenance_operations_APIs(void)
{
	ALT_STATUS_CODE result_code;

	/* Usage:	ALT_STATUS_CODE  alt_mmu_tlb_invalidate (void);	*/
	result_code = alt_mmu_tlb_invalidate();
	if(result_code != ALT_E_SUCCESS){
		printf("ERROR!!: ");
	}
	printf("alt_mmu_tlb_invalidate() .... result=%d\n", (int)result_code);

	/* Usage:	ALT_STATUS_CODE  alt_mmu_tlb_invalidate_is (void);	*/
	result_code = alt_mmu_tlb_invalidate_is();
	if(result_code != ALT_E_SUCCESS){
		printf("ERROR!!: ");
	}
	printf("alt_mmu_tlb_invalidate_is() .... result=%d\n", (int)result_code);


	return;
}

/********************************************************************************//*!
 * @fn  void sample_mmu_print_registers(void)
 * @brief  Print the MMU related Registers.
 *
 * @details  Print the register values obtained using the API and coprocessor instructions.
 * @param[in,out] void  none
 * @retval void  none
 *
 * @attention  nothing.
 * @pre        nothing.
 * @post       nothing.
 * @note       nothing.
 *
 * @date <b> History: \<Date\> \<Rev\> \<Keyword\> \<Details\> </b>
 * @date 2014/03/03  ---  #13288  新規作成.
 **//*******************************************************************************/
void sample_mmu_print_registers(void)
{
	/* The value returned by the following API doesn't register values. */
	/* Retrieve only the value of the base address of the translation table. */

	/* Usage:	void *  alt_mmu_TTBR0_get (void);	*/
	printf("-- Translation Table Base Register 0 (TTBR0) --\n");
	printf("alt_mmu_TTBR0_get() .... result=0x%08X\n", (int)alt_mmu_TTBR0_get());
	/* Usage:	void *  alt_mmu_TTBR1_get (void);	*/
	printf("-- Translation Table Base Register 1 (TTBR1) --\n");
	printf("alt_mmu_TTBR1_get() .... result=0x%08X\n", (int)alt_mmu_TTBR1_get());
	
	/* API to get the following register values not provided in HWLib. */
	uint32_t ttbr0;			/* Translation Table Base Register 0 (TTBR0) */
	uint32_t ttbr1;			/* Translation Table Base Register 1 (TTBR1) */
	uint32_t ttbcr;			/* Translation Table Base Control Register (TTBCR) */
	uint32_t dacr;			/* Domain Access Control Register (DACR) */
	uint32_t contextidr;	/* Context ID Register (CONTEXTIDR) */
	__asm("MRC p15, 0, %0, c2, c0, 0" : "=r" (ttbr0));
	__asm("MRC p15, 0, %0, c2, c0, 1" : "=r" (ttbr1));
	__asm("MRC p15, 0, %0, c2, c0, 2" : "=r" (ttbcr));
	__asm("MRC p15, 0, %0, c3, c0, 0" : "=r" (dacr));
	__asm("MRC p15, 0, %0, c13, c0, 1" : "=r" (contextidr));
	printf("-- TTBRx real values, and the other register values. --\n");
	printf("TTBR0      = 0x%08X\n", (int)ttbr0);
	printf("TTBR1      = 0x%08X\n", (int)ttbr1);
	printf("TTBCR      = 0x%08X\n", (int)ttbcr);
	printf("DACR       = 0x%08X\n", (int)dacr);
	printf("CONTEXTIDR = 0x%08X\n", (int)contextidr);

	return;
}

/********************************************************************************//*!
 * @fn  void sample_mmu_test_register_setget_APIs(void)
 * @brief  Test the API to read and write MMU related register.
 *
 * @details  Test the API name corresponding to the following rules.
 *           - alt_mmu_XXXX_get
 *           - alt_mmu_XXXX_set
 * @param[in,out] void  none
 * @retval void  none
 *
 * @attention  nothing.
 * @pre        nothing.
 * @post       nothing.
 * @note       nothing.
 *
 * @date <b> History: \<Date\> \<Rev\> \<Keyword\> \<Details\> </b>
 * @date 2014/03/03  ---  #13288  新規作成.
 **//*******************************************************************************/
void sample_mmu_test_register_setget_APIs(void)
{
	ALT_STATUS_CODE result_code;
	uint32_t setval_u32, setval_u32_2;
	static ALT_MMU_DAP_t test_domain_ap[16] = {
		ALT_MMU_DAP_MANAGER,	ALT_MMU_DAP_MANAGER,	ALT_MMU_DAP_MANAGER,	ALT_MMU_DAP_MANAGER,
		ALT_MMU_DAP_CLIENT,		ALT_MMU_DAP_CLIENT,		ALT_MMU_DAP_CLIENT,		ALT_MMU_DAP_CLIENT,
		ALT_MMU_DAP_NO_ACCESS,	ALT_MMU_DAP_NO_ACCESS,	ALT_MMU_DAP_NO_ACCESS,	ALT_MMU_DAP_NO_ACCESS,
		ALT_MMU_DAP_MANAGER,	ALT_MMU_DAP_MANAGER,	ALT_MMU_DAP_MANAGER,	ALT_MMU_DAP_MANAGER
	};

	/* Testing is intended. Run the API sets the value of a register. */
	/* This function should do before you start the MMU setting. */

	/* Usage:	ALT_STATUS_CODE  alt_mmu_TTBR0_set (const void *addr);	*/
	setval_u32 = 0x03000000;
	result_code = alt_mmu_TTBR0_set((void*)setval_u32);
	if(result_code != ALT_E_SUCCESS){
		printf("ERROR!!: ");
	}
	printf("alt_mmu_TTBR0_set(0x%08X) .... result=%d\n", (int)setval_u32, (int)result_code);

	/* Usage:	ALT_STATUS_CODE  alt_mmu_TTBR1_set (const void *addr);	*/
	setval_u32 = 0x04000000;
	result_code = alt_mmu_TTBR1_set((void*)setval_u32);
	if(result_code != ALT_E_SUCCESS){
		printf("ERROR!!: ");
	}
	printf("alt_mmu_TTBR1_set(0x%08X) .... result=%d\n", (int)setval_u32, (int)result_code);

	/* Usage:	ALT_STATUS_CODE  alt_mmu_TTBCR_set (const bool enable_ttbr0_walk, const bool enable_ttbr1_walk, const uint32_t base_addr_width);	*/
	result_code = alt_mmu_TTBCR_set(false, false, 4);
	if(result_code != ALT_E_SUCCESS){
		printf("ERROR!!: ");
	}
	printf("alt_mmu_TTBCR_set(false,false,3) .... result=%d\n", (int)result_code);

	/* Usage:	ALT_STATUS_CODE  alt_mmu_DACR_set (const ALT_MMU_DAP_t domain_ap[], const size_t num_elem);	*/
	result_code = alt_mmu_DACR_set(test_domain_ap, 16);
	if(result_code != ALT_E_SUCCESS){
		printf("ERROR!!: ");
	}
	printf("alt_mmu_DACR_set(test_domain_ap,16) .... result=%d\n", (int)result_code);

	/* Usage:	ALT_STATUS_CODE  alt_mmu_CONTEXTIDR_set (const uint32_t procid, const uint32_t asid);	*/
	setval_u32 = 0;
	setval_u32_2 = 0;
	result_code = alt_mmu_CONTEXTIDR_set(setval_u32, setval_u32_2);
	if(result_code != ALT_E_SUCCESS){
		printf("ERROR!!: ");
	}
	printf("alt_mmu_CONTEXTIDR_set(%d,%d) .... result=%d\n", (int)setval_u32, (int)setval_u32_2, (int)result_code);

	sample_mmu_print_registers();
	
	return;
}

/********************************************************************************//*!
 * @fn  void sample_test_init(void) 
 * @brief  Initializing the sample program. 
 * 
 * @details  Call the initialization function for MMU Management API. 
 *           Other, do the following.
 *           -# Configure the Helio Board GPIO settings. 
 *           -# Print Current Settings of ACP ID Mapper. 
 *           -# Test for set/get APIs.
 *           -# API Test for 'alt_mmu_va_space_storage_required'.
 * @param[in,out] void  none 
 * @retval void  none 
 * 
 * @attention  nothing. 
 * @pre        nothing. 
 * @post       nothing. 
 * @note       nothing. 
 * 
 * @date <b> History: \<Date\> \<Rev\> \<Keyword\> \<Details\> </b>
 * @date 2014/03/03  ---  #13288  新規作成. 
 **//*******************************************************************************/
void sample_test_init(void)
{
	ALT_STATUS_CODE result_code;
	size_t trans_table_bytes;
	
	sample_gpio_config_helioboard();

	/* Print Current Settings of ACP ID Mapper. */
	sample_adrspcmng_print_ACPIDMapper();
	
	/* Usage:	ALT_STATUS_CODE  alt_mmu_init (void);	*/
	result_code = alt_mmu_init();
	if(result_code != ALT_E_SUCCESS){
		printf("ERROR!!: ");
	}
	printf("alt_mmu_init() .... result=%d\n", (int)result_code);
	
	/* Test for set/get APIs. */
	sample_mmu_test_register_setget_APIs();

	/* Usage:	size_t  alt_mmu_va_space_storage_required (const ALT_MMU_MEM_REGION_t *mem_regions, const size_t num_mem_regions);	*/
	/************************************************************/
	/* target discriptor : ALT_MMU_MEM_REGION_t MemoryRegion[] 	*/
	/* 						MemoryRegion[0] .. SDRAM			*/
	/* 						MemoryRegion[1] .. SDRAM2			*/
	/* 						MemoryRegion[2] .. SDRAM3 (include .text)	*/
	/* 						MemoryRegion[3] .. HPS2FPGA			*/
	/* 						MemoryRegion[4] .. LW_HPS2FPGA		*/
	/* 						MemoryRegion[5] .. PERIPHRALS		*/
	/* 						MemoryRegion[6] .. BOOTROM			*/
	/* 						MemoryRegion[7] .. MPU_SCU			*/
	/* 						MemoryRegion[8] .. MPU_L2			*/
	/* 						MemoryRegion[9] .. RAM				*/
	/************************************************************/
	trans_table_bytes = alt_mmu_va_space_storage_required(MemoryRegion, 10);
	printf("alt_mmu_va_space_storage_required() ... result=0x%08X(%d) bytes\n", 
		(int)trans_table_bytes, (int)trans_table_bytes);
	
	
	return;
}

/********************************************************************************//*!
 * @fn  void sample_test_uninit(void) 
 * @brief  Uninitializing the sample program. 
 * 
 * @details  Call the uninitialization function for MMU Management API.
 *           Other, do the following.
 *           -# API Test for 'alt_mmu_ttb1_init'. 
 * @param[in,out] void  none 
 * @retval void  none 
 * 
 * @attention  nothing. 
 * @pre        nothing. 
 * @post       nothing. 
 * @note       nothing. 
 * 
 * @date <b> History: \<Date\> \<Rev\> \<Keyword\> \<Details\> </b>
 * @date 2014/03/03  ---  #13288  新規作成. 
 **//*******************************************************************************/
void sample_test_uninit(void)
{
	ALT_STATUS_CODE result_code;
	uint32_t ttb1_base_addr;
	
	/* Usage:	ALT_STATUS_CODE  alt_mmu_ttb1_init (uint32_t *ttb1);	*/
	ttb1_base_addr = (uint32_t)alt_mmu_TTBR0_get();
	result_code = alt_mmu_ttb1_init((uint32_t*)ttb1_base_addr);
	if(result_code != ALT_E_SUCCESS){
		printf("ERROR!!: ");
	}
	printf("alt_mmu_ttb1_init(0x%08X) .... result=%d\n", (int)ttb1_base_addr, (int)result_code);

	/* Usage:	ALT_STATUS_CODE  alt_mmu_uninit (void);	*/
	result_code = alt_mmu_uninit();
	if(result_code != ALT_E_SUCCESS){
		printf("ERROR!!: ");
	}
	printf("alt_mmu_uninit() .... result=%d\n", (int)result_code);
	
	return;
}

/********************************************************************************//*!
 * @fn  void sample_test_print_usage(void) 
 * @brief  Display usage of the sample program. 
 * 
 * @details  Following information displays on the console.
 *            - Overview of HPS for push button functions (or operations)
 *            - Overview of HPS for DIP switch functions (or operations)
 * @param[in,out] void  none 
 * @retval void  none 
 * 
 * @attention  nothing. 
 * @pre        nothing. 
 * @post       nothing. 
 * @note       nothing. 
 * 
 * @date <b> History: \<Date\> \<Rev\> \<Keyword\> \<Details\> </b>
 * @date 2014/08/18  ---  #xxxxx  新規作成. 
 **//*******************************************************************************/
void sample_test_print_usage(void)
{
	printf("+--- Usage ( Function of HPS-PUSHSW and HPS-DIPSW )  -----------+\n");
	printf("+ PUSHSW HPS 0(SW8) .... Exit Test loop!!!\n");
	printf("+ PUSHSW HPS 1(SW10) ... Run the APIs for setting TTBs as testing.\n");
	printf("+ PUSHSW HPS 2(SW7) .... Run the APIs for TLB maintenace as testing.\n");
	printf("+ PUSHSW HPS 3(SW9) .... Display the MMU registers.\n");
	printf("+ DIPSW HPS(SW14) 3 .... Reconfigure the MMU at ON/OFF changing.\n");
	printf("+ DIPSW HPS(SW14) 4 .... Select MMU Disable(OFF) or Enable(ON).\n");
	printf("+---------------------------------------------------------------+\n\n");
	
#if 0
	ループ中にHPS用PUSHSWの操作を検出した場合、それぞれ以下の処理を実行します。
		- HPS用DIPSW4 .... OFF切替：MMU無効化(Disable)、ON切替：MMU有効化(Enable)
		- HPS用DIPSW3 .... ON/OFF切替：MMU再初期化（Translation Tableを再作成しMMU有効化）
		- HPS用PUSHSW0 ... ループを抜けてプログラム終了します。(uninit処理を実行)
		- HPS用PUSHSW1 ... Translation Tableのディスクリプタ書換え用APIを試行。
		- HPS用PUSHSW2 ... TLBのメンテナンス操作用APIを試行。
		- HPS用PUSHSW3 ... MMU関連レジスタの現在値を表示。
#endif
	
	return;
}

/********************************************************************************//*!
 * @fn  void sample_test_main(void) 
 * @brief  Main routine of the sample program. 
 * 
 * @details  Infinite loop structure works. Make the detection of various types 
 *           of switches in the loop, it performs in various testing processes.
 * @param[in,out] void  none 
 * @retval void  none 
 * 
 * @attention  nothing. 
 * @pre        nothing. 
 * @post       nothing. 
 * @note       nothing. 
 * 
 * @date <b> History: \<Date\> \<Rev\> \<Keyword\> \<Details\> </b>
 * @date 2014/03/03  ---  #13288  新規作成. 
 **//*******************************************************************************/
void sample_test_main(void)
{
	uint32_t ui32portc = 0;
	uint32_t ui32portcbk = 0;
	uint32_t ui32portc_onchg = 0;
	uint32_t ui32dipsw = 0;

	/* ################################################ */
	/* ## Implement the test setting process here!!! ## */
	/* ################################################ */

	/* Setting for the Address Space Remap Register. */
	sample_adrspcmng_setting_remap();
	/* Setting for the L2 Cache Address Filter. */
	sample_adrspcmng_setting_l2addrfilter(0, 0xC0000000);

	// Setting TEST Data!!! //
	/*! SDRAM .. PA:0x00000000~0x003FFFFF <--> VA:0x00000000~0x003FFFFF */
	/*! SDRAM2 .. PA:0x00800000~0x00BFFFFF <--> VA:0x00400000~0x007FFFFF */
	printf("Setting TEST Data : 0x00000000~0x0000FFFF.\n");
	sample_dmac_setting_testdata((uint32_t*)0, (uint32_t*)0x0000FFFF);
	printf("Setting TEST Data : 0x00200000~0x0020FFFF.\n");
	sample_dmac_setting_testdata((uint32_t*)0x00200000, (uint32_t*)0x0020FFFF);
	printf("Setting TEST Data : 0x002F0000~0x002FFFFF.\n");
	sample_dmac_setting_testdata((uint32_t*)0x002F0000, (uint32_t*)0x002FFFFF);
	printf("Setting TEST Data : 0x00300000~0x0030FFFF.\n");
	sample_dmac_setting_testdata((uint32_t*)0x00300000, (uint32_t*)0x0030FFFF);
	printf("Setting TEST Data : 0x003F0000~0x003FFFFF.\n");
	sample_dmac_setting_testdata((uint32_t*)0x003F0000, (uint32_t*)0x003FFFFF);
	printf("Setting TEST Data : 0x00400000~0x0040FFFF.\n");
	sample_dmac_setting_testdata((uint32_t*)0x00400000, (uint32_t*)0x0040FFFF);
	printf("Setting TEST Data : 0x004F0000~0x004FFFFF.\n");
	sample_dmac_setting_testdata((uint32_t*)0x004F0000, (uint32_t*)0x004FFFFF);
	printf("Setting TEST Data : 0x00500000~0x0050FFFF.\n");
	sample_dmac_setting_testdata((uint32_t*)0x00500000, (uint32_t*)0x0050FFFF);
	printf("Setting TEST Data : 0x005F0000~0x005FFFFF.\n");
	sample_dmac_setting_testdata((uint32_t*)0x005F0000, (uint32_t*)0x005FFFFF);
	printf("Setting TEST Data : 0x00600000~0x0060FFFF.\n");
	sample_dmac_setting_testdata((uint32_t*)0x00600000, (uint32_t*)0x0060FFFF);
	printf("Setting TEST Data : 0x006F0000~0x006FFFFF.\n");
	sample_dmac_setting_testdata((uint32_t*)0x006F0000, (uint32_t*)0x006FFFFF);
	printf("Setting TEST Data : 0x00700000~0x0070FFFF.\n");
	sample_dmac_setting_testdata((uint32_t*)0x00700000, (uint32_t*)0x0070FFFF);
	printf("Setting TEST Data : 0x007F0000~0x007FFFFF.\n");
	sample_dmac_setting_testdata((uint32_t*)0x007F0000, (uint32_t*)0x007FFFFF);
	printf("Setting TEST Data : 0x00800000~0x0080FFFF.\n");
	sample_dmac_setting_testdata((uint32_t*)0x00800000, (uint32_t*)0x0080FFFF);
	printf("Setting TEST Data : 0x008F0000~0x008FFFFF.\n");
	sample_dmac_setting_testdata((uint32_t*)0x008F0000, (uint32_t*)0x008FFFFF);
	printf("Setting TEST Data : 0x00900000~0x0090FFFF.\n");
	sample_dmac_setting_testdata((uint32_t*)0x00900000, (uint32_t*)0x0090FFFF);
	printf("Setting TEST Data : 0x009F0000~0x009FFFFF.\n");
	sample_dmac_setting_testdata((uint32_t*)0x009F0000, (uint32_t*)0x009FFFFF);
	printf("Setting TEST Data : 0x00A00000~0x00A0FFFF.\n");
	sample_dmac_setting_testdata((uint32_t*)0x00A00000, (uint32_t*)0x00A0FFFF);
	printf("Setting TEST Data : 0x00AF0000~0x00AFFFFF.\n");
	sample_dmac_setting_testdata((uint32_t*)0x00AF0000, (uint32_t*)0x00AFFFFF);
	printf("Setting TEST Data : 0x00B00000~0x00B0FFFF.\n");
	sample_dmac_setting_testdata((uint32_t*)0x00B00000, (uint32_t*)0x00B0FFFF);
	printf("Setting TEST Data : 0x00BF0000~0x00BFFFFF.\n");
	sample_dmac_setting_testdata((uint32_t*)0x00BF0000, (uint32_t*)0x00BFFFFF);
	printf("Setting TEST Data : 0x02000000~0x0200FFFF.\n");
	sample_dmac_setting_testdata((uint32_t*)0x02000000, (uint32_t*)0x0200FFFF);
	printf("Setting TEST Data : 0x03000000~0x0300FFFF.\n");
	sample_dmac_setting_testdata((uint32_t*)0x03000000, (uint32_t*)0x0300FFFF);
	printf("Setting TEST Data : 0x03FF0000~0x03FFFFFF.\n");
	sample_dmac_setting_testdata((uint32_t*)0x03FF0000, (uint32_t*)0x03FFFFFF);
	printf("Setting TEST Data : 0xFFFF0000~0xFFFFFFFF.\n");
	sample_dmac_setting_testdata((uint32_t*)0xFFFF0000, (uint32_t*)0xFFFFFFFF);

	printf("MMU Virtual Address Setting & Enable.\n");
	sample_mmu_init_MMU();
	sample_mmu_print_registers();


	printf("==== Start While(1) loop process!!! (Exit PUSHSW0(SW8) becomes ON.) ====\n");
	sample_test_print_usage();
	while(1)
	{
		// ---- Check the DIPSW and PUSHSW. ----
		ui32portc = alt_gpio_port_data_read(ALT_GPIO_PORTC, ALT_GPIO_PORTBIT_SWALL);
		ui32portc = ~ui32portc & ALT_GPIO_PORTBIT_SWALL;	// bit inversion and mask ({ON=0,OFF=1}->{ON=1,OFF=0})
		ui32portc_onchg |= (ui32portc ^ ui32portcbk) & ui32portc;

		// DIPSW HPS 1-4
		if((ui32portcbk ^ ui32portc) & ALT_GPIO_PORTBIT_DIPSWALL)
		{
			// Detect Time Measurement Index from DIPSW1-4 Value.
			ui32dipsw = (uint32_t)((ui32portc & ALT_GPIO_PORTBIT_DIPSWALL) >> ALT_GPIO_LSBBIT_DIPSWS);
			ui32dipsw *= 1;		// To avoid warnings.

			/* ######################################## */
			/* ## Implement the test process here!!! ## */
			/* ######################################## */
			if((ui32portcbk ^ ui32portc) & ALT_GPIO_PORTBIT_DIPS1){
			}
			if((ui32portcbk ^ ui32portc) & ALT_GPIO_PORTBIT_DIPS2){
			}
			if((ui32portcbk ^ ui32portc) & ALT_GPIO_PORTBIT_DIPS3){
				printf("MMU Virtual Address Re-Setting & Enable.\n");
				sample_mmu_init_MMU();
			}
			if((ui32portcbk ^ ui32portc) & ALT_GPIO_PORTBIT_DIPS4){
				if(ui32portc & ALT_GPIO_PORTBIT_DIPS4){
					printf("MMU Enable.\n");
					alt_mmu_enable();
				} else {
					printf("MMU Disable.\n");
					alt_mmu_disable();
				}
			}
		}

		// PUSHSW HPS 0
		if(ui32portc_onchg & ALT_GPIO_PORTBIT_PUSH0)
		{
			ui32portc_onchg &= ~ALT_GPIO_PORTBIT_PUSH0;
			
			/* ######################################## */
			/* ## Implement the test process here!!! ## */
			/* ######################################## */
			break;	// Exit PUSHSW0(SW8) becomes ON.
			
		}
		// PUSHSW HPS 1
		if(ui32portc_onchg & ALT_GPIO_PORTBIT_PUSH1)
		{
			ui32portc_onchg &= ~ALT_GPIO_PORTBIT_PUSH1;
			
			/* ######################################## */
			/* ## Implement the test process here!!! ## */
			/* ######################################## */
			printf("For the tests to customize TTB.\n");
			sample_mmu_test_ttb_desc_set_APIs();
		}
		// PUSHSW HPS 2
		if(ui32portc_onchg & ALT_GPIO_PORTBIT_PUSH2)
		{
			ui32portc_onchg &= ~ALT_GPIO_PORTBIT_PUSH2;
			
			/* ######################################## */
			/* ## Implement the test process here!!! ## */
			/* ######################################## */
			printf("Invalidate the entire unified TLB.\n");
			sample_mmu_test_TLB_maintenance_operations_APIs();
		}
		// PUSHSW HPS 3
		if(ui32portc_onchg & ALT_GPIO_PORTBIT_PUSH3)
		{
			ui32portc_onchg &= ~ALT_GPIO_PORTBIT_PUSH3;
			
			/* ######################################## */
			/* ## Implement the test process here!!! ## */
			/* ######################################## */
			sample_mmu_print_registers();
		}

		ui32portcbk = ui32portc;
	}
	printf("==== End While(1) loop process. (Exit PUSHSW0(SW8) becomes ON.) ====\n");

	printf("MMU Disable.\n");
	alt_mmu_disable();

	return;
}

// All APIs to be tested in this sample.
/* MMU Management API */
/* +-- MMU Management */
/* Usage:	ALT_STATUS_CODE  alt_mmu_init (void);	*/
/* Usage:	ALT_STATUS_CODE  alt_mmu_uninit (void);	*/
/* Usage:	ALT_STATUS_CODE  alt_mmu_ttb1_init (uint32_t *ttb1);	*/
/* Usage:	ALT_STATUS_CODE  alt_mmu_ttb1_desc_set (uint32_t *ttb1, const void *va, const uint32_t desc);	*/
/* Usage:	ALT_STATUS_CODE  alt_mmu_ttb2_desc_set (const uint32_t *ttb1, const void *va, const uint32_t desc);	*/
/* Usage:	ALT_STATUS_CODE  alt_mmu_disable (void);	*/
/* Usage:	ALT_STATUS_CODE  alt_mmu_enable (void);	*/
/* Usage:	void *  alt_mmu_TTBR0_get (void);	*/
/* Usage:	ALT_STATUS_CODE  alt_mmu_TTBR0_set (const void *addr);	*/
/* Usage:	void *  alt_mmu_TTBR1_get (void);	*/
/* Usage:	ALT_STATUS_CODE  alt_mmu_TTBR1_set (const void *addr);	*/
/* Usage:	ALT_STATUS_CODE  alt_mmu_TTBCR_set (const bool enable_ttbr0_walk, const bool enable_ttbr1_walk, const uint32_t base_addr_width);	*/
/* Usage:	ALT_STATUS_CODE  alt_mmu_DACR_set (const ALT_MMU_DAP_t domain_ap[], const size_t num_elem);	*/
/* Usage:	ALT_STATUS_CODE  alt_mmu_CONTEXTIDR_set (const uint32_t procid, const uint32_t asid);	*/
/* Usage:	ALT_STATUS_CODE  alt_mmu_tlb_invalidate (void);	*/
/* Usage:	ALT_STATUS_CODE  alt_mmu_tlb_invalidate_is (void);	*/
/* +-- MMU Virtual Address Space Creation */
/* Usage:	size_t  alt_mmu_va_space_storage_required (const ALT_MMU_MEM_REGION_t *mem_regions, const size_t num_mem_regions);	*/
/* Usage:	ALT_STATUS_CODE  alt_mmu_va_space_create (uint32_t **ttb1, const ALT_MMU_MEM_REGION_t *mem_regions, const size_t num_mem_regions, alt_mmu_ttb_alloc_t ttb_alloc, void *ttb_alloc_context);	*/
/* Usage:	ALT_STATUS_CODE  alt_mmu_va_space_enable (const uint32_t *ttb1);	*/

////////////////////////////// Common Functions //////////////////////////////////////// 
/********************************************************************************//*!
 * @fn  static void sample_gpio_config_helioboard(void) 
 * @brief  Configure the Helio Board GPIO settings. 
 * 
 * @details  Performs the following processing. 
 *           - Gets the contents of the current port settings to gpio_config_array. 
 *           - Global timer counter value of the measurement at the start
 *           - Setting DIPSW and PUSHSW via bit index configuration func. 
 * @param[in,out] void  none 
 * @retval void  none 
 * 
 * @attention  nothing. 
 * @pre        nothing. 
 * @post       nothing. 
 * @note       nothing. 
 * 
 * @date <b> History: \<Date\> \<Rev\> \<Keyword\> \<Details\> </b>
 * @date 2014/02/13  ---  #13144  新規作成. 
 **//*******************************************************************************/
static void sample_gpio_config_helioboard(void)
{
	ALT_STATUS_CODE result_code;
	ALT_GPIO_CONFIG_RECORD_t gpio_config_array[10];
	ALT_GPIO_CONFIG_RECORD_t* p_gpiocfg;

	// Setting OUT-PORT（GPIO61） via normal configuration func.
	/****************************************************************************************/
	/* Usage: ALT_STATUS_CODE alt_gpio_port_config(	ALT_GPIO_PORT_t gpio_pid,
	 * 												uint32_t mask,
	 * 												ALT_GPIO_PIN_DIR_t dir,
	 * 												ALT_GPIO_PIN_TYPE_t type,
	 * 												ALT_GPIO_PIN_POL_t pol,
	 * 												ALT_GPIO_PIN_DEBOUNCE_t debounc,
	 * 												uint32_t data); 						*/
	/****************************************************************************************/
	result_code = alt_gpio_port_config(ALT_GPIO_PORTC, ALT_GPIO_BIT3,
			ALT_GPIO_PIN_OUTPUT, ALT_GPIO_PIN_LEVEL_TRIG_INT, ALT_GPIO_PIN_ACTIVE_HIGH, ALT_GPIO_PIN_NODEBOUNCE,
			ALT_GPIO_BIT3);
	if(result_code != ALT_E_SUCCESS)
	{
		printf("ERROR!!: alt_gpio_port_config() result=%d\n", (int)result_code);
	}

	// Gets the contents of the current port settings to gpio_config_array.
	/****************************************************************************************/
	/* Usage: ALT_STATUS_CODE alt_gpio_group_config_get(ALT_GPIO_CONFIG_RECORD_t* config_array, uint32_t len);	*/
	/****************************************************************************************/
	gpio_config_array[0].signal_number = ALT_GPIO_1BIT_61;
	gpio_config_array[1].signal_number = ALT_HLGPI_6;
	gpio_config_array[2].signal_number = ALT_HLGPI_7;
	gpio_config_array[3].signal_number = ALT_HLGPI_8;
	gpio_config_array[4].signal_number = ALT_HLGPI_9;
	gpio_config_array[5].signal_number = ALT_HLGPI_10;
	gpio_config_array[6].signal_number = ALT_HLGPI_11;
	gpio_config_array[7].signal_number = ALT_HLGPI_12;
	gpio_config_array[8].signal_number = ALT_HLGPI_13;
	gpio_config_array[9].signal_number = ALT_END_OF_GPIO_SIGNALS;
	result_code = alt_gpio_group_config_get(gpio_config_array, 10);
	if(result_code != ALT_E_SUCCESS)
	{
		printf("ERROR!!: alt_gpio_group_config_get() result=%d\n", (int)result_code);
	}

	// Setting DIPSW and PUSHSW via bit index configuration func.
	/****************************************************************************************/
	/* Usage: ALT_STATUS_CODE alt_gpio_group_config(ALT_GPIO_CONFIG_RECORD_t* config_array, uint32_t len); */
	/****************************************************************************************/
	for(int i=1; i<10; i++)
	{
		p_gpiocfg = &gpio_config_array[i];
		p_gpiocfg->direction = ALT_GPIO_PIN_INPUT;
		p_gpiocfg->type = ALT_GPIO_PIN_LEVEL_TRIG_INT;
		p_gpiocfg->polarity = ALT_GPIO_PIN_ACTIVE_HIGH;
		p_gpiocfg->debounce = ALT_GPIO_PIN_NODEBOUNCE;
		p_gpiocfg->data = ALT_GPIO_PIN_DATAZERO;
	}
	result_code = alt_gpio_group_config(gpio_config_array, 10);
	if(result_code != ALT_E_SUCCESS)
	{
		printf("ERROR!!: alt_gpio_group_config() result=%d\n", (int)result_code);
	}

	return;
}

/********************************************************************************//*!
 * @fn  static void sample_dmac_setting_testdata(uint32_t* start, uint32_t* endnext)
 * @brief  Writes the data for transfer test.
 *
 * @details  Writes the address value area of memory that is specified for the transfer test.
 * @param[in] start  start address
 * @param[in] endnext  Next to the end address
 * @retval void  none
 *
 * @attention  nothing.
 * @pre        nothing.
 * @post       nothing.
 * @note       nothing.
 *
 * @date <b> History: \<Date\> \<Rev\> \<Keyword\> \<Details\> </b>
 * @date 2014/02/26  ---  #13246  新規作成.
 **//*******************************************************************************/
static void sample_dmac_setting_testdata(uint32_t* start, uint32_t* endnext)
{
	while(start < endnext){
		*start = (uint32_t)start;
		start++;
		if(start < (uint32_t*)0x00000004)	break;
	}

	return;
}

/***********************************************************************************
 * end of file 
 ***********************************************************************************/
